@@ -1,9 +1,9 @@
 /*
- * "$Id: scsi.c,v 1.4 2002/05/24 21:04:47 mike Exp $"
+ * "$Id: scsi.c,v 1.6 2002/12/17 18:56:34 swdev Exp $"
  *
  *   SCSI printer backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 2002 by Easy Software Products, all rights reserved.
+ *   Copyright 2003 by Easy Software Products, all rights reserved.
  *
  *   Redistribution and use in source and binary forms, with or
  *   without modification, are permitted provided that the
@@ -113,6 +113,9 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   int		fp;		/* Print file */
   int		copies;		/* Number of copies to print */
   int		status;		/* Exit status */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  struct sigaction action;	/* Actions for POSIX signals */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
  /*
@@ -121,6 +124,20 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   setbuf(stderr, NULL);
 
+ /*
+  * Ignore SIGPIPE signals...
+  */
+
+#ifdef HAVE_SIGSET
+  sigset(SIGPIPE, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+  action.sa_handler = SIG_IGN;
+  sigaction(SIGPIPE, &action, NULL);
+#else
+  signal(SIGPIPE, SIG_IGN);
+#endif /* HAVE_SIGSET */
+
  /*
   * Check command-line...
   */
@@ -202,5 +219,5 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
 
 /*
- * End of "$Id: scsi.c,v 1.4 2002/05/24 21:04:47 mike Exp $".
+ * End of "$Id: scsi.c,v 1.6 2002/12/17 18:56:34 swdev Exp $".
  */
