@@ -1,5 +1,5 @@
 /*
- * "$Id: socket.c,v 1.32 2002/12/17 18:56:34 swdev Exp $"
+ * "$Id: socket.c,v 1.37 2003/04/10 12:57:40 mike Exp $"
  *
  *   AppSocket backend for the Common UNIX Printing System (CUPS).
  *
@@ -54,6 +54,46 @@
 #endif /* WIN32 */
 
 
+/*
+ * Some OS's don't have hstrerror(), most notably Solaris...
+ */
+
+#ifndef HAVE_HSTRERROR
+#  define hstrerror cups_hstrerror
+
+const char *					/* O - Error string */
+cups_hstrerror(int error)			/* I - Error number */
+{
+  static const char * const errors[] =
+		{
+		  "OK",
+		  "Host not found.",
+		  "Try again.",
+		  "Unrecoverable lookup error.",
+		  "No data associated with name."
+		};
+
+
+  if (error < 0 || error > 4)
+    return ("Unknown hostname lookup error.");
+  else
+    return (errors[error]);
+}
+#elif defined(_AIX)
+/*
+ * AIX doesn't provide a prototype but does provide the function...
+ */
+extern const char *hstrerror(int);
+#endif /* !HAVE_HSTRERROR */
+
+
+/*
+ * Local functions...
+ */
+
+void	print_backchannel(const unsigned char *buffer, int nbytes);
+
+
 /*
  * 'main()' - Send a file to the printer or server.
  *
@@ -62,9 +102,9 @@
  *    printer-uri job-id user title copies options [file]
  */
 
-int			/* O - Exit status */
-main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
-     char *argv[])	/* I - Command-line arguments */
+int				/* O - Exit status */
+main(int  argc,			/* I - Number of command-line arguments (6 or 7) */
+     char *argv[])		/* I - Command-line arguments */
 {
   char		method[255],	/* Method in URI */
 		hostname[1024],	/* Hostname */
@@ -79,8 +119,8 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   struct sockaddr_in addr;	/* Socket address */
   struct hostent *hostaddr;	/* Host address */
   int		wbytes;		/* Number of bytes written */
-  size_t	nbytes,		/* Number of bytes read */
-		tbytes;		/* Total number of bytes written */
+  int		nbytes;		/* Number of bytes read */
+  size_t	tbytes;		/* Total number of bytes written */
   char		buffer[8192],	/* Output buffer */
 		*bufptr;	/* Pointer into buffer */
   struct timeval timeout;	/* Timeout for select() */
@@ -167,7 +207,7 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   if ((hostaddr = httpGetHostByName(hostname)) == NULL)
   {
     fprintf(stderr, "ERROR: Unable to locate printer \'%s\' - %s\n",
-            hostname, strerror(errno));
+            hostname, hstrerror(h_errno));
     return (1);
   }
 
@@ -282,19 +322,18 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
       FD_ZERO(&input);
       FD_SET(fd, &input);
-#ifdef __hpux
-      if (select(fd + 1, (int *)&input, NULL, NULL, &timeout) > 0)
-#else
       if (select(fd + 1, &input, NULL, NULL, &timeout) > 0)
-#endif /* __hpux */
       {
        /*
 	* Grab the data coming back and spit it out to stderr...
 	*/
 
 	if ((nbytes = recv(fd, buffer, sizeof(buffer), 0)) > 0)
-	  fprintf(stderr, "INFO: Received %lu bytes of back-channel data!\n",
-	          (unsigned long)nbytes);
+	{
+	  fprintf(stderr, "INFO: Received %d bytes of back-channel data!\n",
+	          nbytes);
+          print_backchannel((unsigned char *)buffer, nbytes);
+        }
       }
       else if (argc > 6)
 	fprintf(stderr, "INFO: Sending print file, %lu bytes...\n",
@@ -333,9 +372,12 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 	*/
 
 	if ((nbytes = recv(fd, buffer, sizeof(buffer), 0)) > 0)
-	  fprintf(stderr, "INFO: Received %lu bytes of back-channel data!\n",
-	          (unsigned long)nbytes);
-        else
+	{
+	  fprintf(stderr, "INFO: Received %d bytes of back-channel data!\n",
+	          nbytes);
+          print_backchannel((unsigned char *)buffer, nbytes);
+        }
+	else
 	  break;
       }
       else
@@ -363,5 +405,43 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
 
 /*
- * End of "$Id: socket.c,v 1.32 2002/12/17 18:56:34 swdev Exp $".
+ * 'print_backchannel()' - Print the contents of a back-channel buffer.
+ */
+
+void
+print_backchannel(const unsigned char *buffer,	/* I - Data buffer */
+                  int                 nbytes)	/* I - Number of bytes */
+{
+  char	line[255],				/* Formatted line */
+	*lineptr;				/* Pointer into line */
+
+
+  for (lineptr = line; nbytes > 0; buffer ++, nbytes --)
+  {
+    if (*buffer < 0x20 || *buffer >= 0x7f)
+    {
+      snprintf(lineptr, sizeof(line) - (lineptr - line), "<%02X>", *buffer);
+      lineptr += strlen(lineptr);
+    }
+    else
+      *lineptr++ = *buffer;
+
+    if ((lineptr - line) > 72)
+    {
+      *lineptr = '\0';
+      fprintf(stderr, "DEBUG: DATA: %s\n", line);
+      lineptr = line;
+    }
+  }
+
+  if (lineptr > line)
+  {
+    *lineptr = '\0';
+    fprintf(stderr, "DEBUG: DATA: %s\n", line);
+  }
+}
+
+
+/*
+ * End of "$Id: socket.c,v 1.37 2003/04/10 12:57:40 mike Exp $".
  */
