@@ -1,5 +1,5 @@
 /*
- * "$Id: lpr.c,v 1.27 2002/12/17 18:56:35 swdev Exp $"
+ * "$Id: lpr.c,v 1.30 2003/05/09 18:50:41 mike Exp $"
  *
  *   "lpr" command for the Common UNIX Printing System (CUPS).
  *
@@ -104,12 +104,12 @@ main(int  argc,		/* I - Number of command-line arguments */
       switch (ch = argv[i][1])
       {
         case 'E' : /* Encrypt */
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
 	    cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
 #else
             fprintf(stderr, "%s: Sorry, no encryption support compiled in!\n",
 	            argv[0]);
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 	    break;
 
 	case '1' : /* TROFF font set 1 */
@@ -307,23 +307,25 @@ main(int  argc,		/* I - Number of command-line arguments */
     if (num_dests == 0)
       num_dests = cupsGetDests(&dests);
 
-    for (j = 0, dest = dests; j < num_dests; j ++, dest ++)
-      if (dest->is_default)
-      {
-	printer = dests[j].name;
+    if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+    {
+      printer = dest->name;
 
-	for (j = 0; j < dest->num_options; j ++)
-	  if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
-	    num_options = cupsAddOption(dest->options[j].name,
-		                        dest->options[j].value,
-					num_options, &options);
-        break;
-      }
+      for (j = 0; j < dest->num_options; j ++)
+	if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
+	  num_options = cupsAddOption(dest->options[j].name,
+		                      dest->options[j].value,
+				      num_options, &options);
+    }
   }
 
   if (printer == NULL)
   {
-    fputs("lpr: error - no default destination available.\n", stderr);
+    if (cupsLastError() >= IPP_BAD_REQUEST)
+      fputs("lpr: error - scheduler not responding!\n", stderr);
+    else
+      fputs("lpr: error - no default destination available.\n", stderr);
+
     return (1);
   }
 
@@ -429,5 +431,5 @@ sighandler(int s)	/* I - Signal number */
 
 
 /*
- * End of "$Id: lpr.c,v 1.27 2002/12/17 18:56:35 swdev Exp $".
+ * End of "$Id: lpr.c,v 1.30 2003/05/09 18:50:41 mike Exp $".
  */
