@@ -1,5 +1,5 @@
 /*
- * "$Id: admin.c,v 1.40 2002/12/17 18:56:36 swdev Exp $"
+ * "$Id: admin.c,v 1.44 2003/04/11 02:11:14 mike Exp $"
  *
  *   Administration CGI for the Common UNIX Printing System (CUPS).
  *
@@ -233,7 +233,7 @@ do_am_class(http_t      *http,		/* I - HTTP connection */
 
       if ((response = cupsDoRequest(http, request, "/")) != NULL)
       {
-	ippSetCGIVars(response, NULL, NULL);
+	ippSetCGIVars(response, NULL, NULL, NULL, 0);
 	ippDelete(response);
       }
 
@@ -548,7 +548,7 @@ do_am_printer(http_t      *http,	/* I - HTTP connection */
       */
 
       if (oldinfo)
-	ippSetCGIVars(oldinfo, NULL, NULL);
+	ippSetCGIVars(oldinfo, NULL, NULL, NULL, 0);
 
       cgiCopyTemplateLang(stdout, TEMPLATES, "modify-printer.tmpl", getenv("LANG"));
     }
@@ -610,7 +610,7 @@ do_am_printer(http_t      *http,	/* I - HTTP connection */
 
     if ((response = cupsDoRequest(http, request, "/")) != NULL)
     {
-      ippSetCGIVars(response, NULL, NULL);
+      ippSetCGIVars(response, NULL, NULL, NULL, 0);
       ippDelete(response);
     }
 
@@ -776,7 +776,7 @@ do_am_printer(http_t      *http,	/* I - HTTP connection */
 
         strlcpy(make, var, sizeof(make));
 
-        ippSetCGIVars(response, "ppd-make", make);
+        ippSetCGIVars(response, "ppd-make", make, NULL, 0);
 	cgiCopyTemplateLang(stdout, TEMPLATES, "choose-model.tmpl",
 	                    getenv("LANG"));
       }
@@ -916,6 +916,7 @@ do_config_printer(http_t      *http,	/* I - HTTP connection */
   ppd_file_t	*ppd;			/* PPD file */
   ppd_group_t	*group;			/* Option group */
   ppd_option_t	*option;		/* Option */
+  ppd_attr_t	*protocol;		/* cupsProtocol attribute */
 
 
  /*
@@ -1153,6 +1154,45 @@ do_config_printer(http_t      *http,	/* I - HTTP connection */
       ippDelete(response);
     }
 
+   /*
+    * Binary protocol support...
+    */
+
+    if (ppd->protocols && strstr(ppd->protocols, "BCP"))
+    {
+      protocol = ppdFindAttr(ppd, "cupsProtocol", NULL);
+
+      cgiSetVariable("GROUP", "PS Binary Protocol");
+      cgiCopyTemplateLang(stdout, TEMPLATES, "option-header.tmpl",
+                          getenv("LANG"));
+
+      cgiSetSize("CHOICES", 2);
+      cgiSetSize("TEXT", 2);
+      cgiSetArray("CHOICES", 0, "None");
+      cgiSetArray("TEXT", 0, "None");
+
+      if (strstr(ppd->protocols, "TBCP"))
+      {
+	cgiSetArray("CHOICES", 1, "TBCP");
+	cgiSetArray("TEXT", 1, "TBCP");
+      }
+      else
+      {
+	cgiSetArray("CHOICES", 1, "BCP");
+	cgiSetArray("TEXT", 1, "BCP");
+      }
+
+      cgiSetVariable("KEYWORD", "protocol");
+      cgiSetVariable("KEYTEXT", "PS Binary Protocol");
+      cgiSetVariable("DEFCHOICE", protocol ? protocol->value : "None");
+
+      cgiCopyTemplateLang(stdout, TEMPLATES, "option-pickone.tmpl",
+	                  getenv("LANG"));
+
+      cgiCopyTemplateLang(stdout, TEMPLATES, "option-trailer.tmpl",
+                          getenv("LANG"));
+    }
+
     cgiCopyTemplateLang(stdout, TEMPLATES, "config-printer2.tmpl",
                         getenv("LANG"));
   }
@@ -1176,7 +1216,9 @@ do_config_printer(http_t      *http,	/* I - HTTP connection */
 
     while (get_line(line, sizeof(line), in) != NULL)
     {
-      if (strncmp(line, "*Default", 8) != 0)
+      if (!strncmp(line, "*cupsProtocol:", 14) && cgiGetVariable("protocol"))
+        continue;
+      else if (strncmp(line, "*Default", 8))
         fprintf(out, "%s\n", line);
       else
       {
@@ -1204,6 +1246,9 @@ do_config_printer(http_t      *http,	/* I - HTTP connection */
       }
     }
 
+    if ((var = cgiGetVariable("protocol")) != NULL)
+      fprintf(out, "*cupsProtocol: %s\n", cgiGetVariable("protocol"));
+
     fclose(in);
     fclose(out);
     close(outfd);
@@ -1547,5 +1592,5 @@ get_line(char *buf,	/* I - Line buffer */
 
 
 /*
- * End of "$Id: admin.c,v 1.40 2002/12/17 18:56:36 swdev Exp $".
+ * End of "$Id: admin.c,v 1.44 2003/04/11 02:11:14 mike Exp $".
  */
