@@ -1,5 +1,5 @@
 /*
- * "$Id: ipp-var.h,v 1.7 2002/12/17 18:56:36 swdev Exp $"
+ * "$Id: ipp-var.h,v 1.9 2003/04/08 03:45:14 mike Exp $"
  *
  *   IPP variable definitions for the Common UNIX Printing System (CUPS).
  *
@@ -26,7 +26,6 @@
  * Include necessary headers...
  */
 
-#include <ctype.h>
 #include <cups/cups.h>
 #include <cups/debug.h>
 #include <cups/language.h>
@@ -47,9 +46,10 @@
 
 extern char	*ippGetTemplateDir(void);
 extern void	ippSetServerVersion(void);
-extern void	ippSetCGIVars(ipp_t *, const char *, const char *);
+extern void	ippSetCGIVars(ipp_t *, const char *, const char *,
+		              const char *, int);
 
 
 /*
- * End of "$Id: ipp-var.h,v 1.7 2002/12/17 18:56:36 swdev Exp $".
+ * End of "$Id: ipp-var.h,v 1.9 2003/04/08 03:45:14 mike Exp $".
  */
