@@ -1,5 +1,5 @@
 /*
- * "$Id: emit.c,v 1.29 2002/12/17 18:56:41 swdev Exp $"
+ * "$Id: emit.c,v 1.32 2003/05/25 14:39:17 mike Exp $"
  *
  *   PPD code emission routines for the Common UNIX Printing System (CUPS).
  *
@@ -196,26 +196,52 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
       * Send DSC comments with option...
       */
 
-      if (fprintf(fp, "%%%%BeginFeature: *%s %s\n",
-                  ((ppd_option_t *)choices[i]->option)->keyword,
-		  choices[i]->choice) < 0)
-      {
-        free(choices);
-        return (-1);
-      }
-
       if ((strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageSize") == 0 ||
            strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageRegion") == 0) &&
           strcasecmp(choices[i]->choice, "Custom") == 0)
       {
        /*
-        * Variable size; write out standard size options (this should
-	* eventually be changed to use the parameter positions defined
-	* in the PPD file...)
+        * Variable size; write out standard size options, using the
+	* parameter positions defined in the PPD file...
 	*/
 
+        ppd_attr_t	*attr;		/* PPD attribute */
+	int		pos,		/* Position of custom value */
+			values[5];	/* Values for custom command */
+
+
+        fputs("%%BeginFeature: *CustomPageSize True\n", fp);
+
         size = ppdPageSize(ppd, "Custom");
-        fprintf(fp, "%.0f %.0f 0 0 0\n", size->width, size->length);
+
+        memset(values, 0, sizeof(values));
+
+	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize", "Width")) != NULL)
+	{
+	  pos = atoi(attr->value) - 1;
+
+          if (pos < 0 || pos > 4)
+	    pos = 0;
+	}
+	else
+	  pos = 0;
+
+	values[pos] = (int)size->width;
+
+	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize", "Height")) != NULL)
+	{
+	  pos = atoi(attr->value) - 1;
+
+          if (pos < 0 || pos > 4)
+	    pos = 1;
+	}
+	else
+	  pos = 1;
+
+	values[pos] = (int)size->length;
+
+        fprintf(fp, "%d %d %d %d %d\n", values[0], values[1],
+	        values[2], values[3], values[4]);
 
 	if (choices[i]->code == NULL)
 	{
@@ -228,6 +254,13 @@ ppdEmit(ppd_file_t    *ppd,		/* I - PPD file record */
 	  fputs(ppd_custom_code, fp);
 	}
       }
+      else if (fprintf(fp, "%%%%BeginFeature: *%s %s\n",
+                       ((ppd_option_t *)choices[i]->option)->keyword,
+		       choices[i]->choice) < 0)
+      {
+        free(choices);
+        return (-1);
+      }
 
       if (choices[i]->code != NULL && choices[i]->code[0] != '\0')
       {
@@ -274,7 +307,8 @@ ppdEmitFd(ppd_file_t    *ppd,		/* I - PPD file record */
           ppd_section_t section)	/* I - Section to write */
 {
   int		i,			/* Looping var */
-		count;			/* Number of choices */
+		count,			/* Number of choices */
+		custom_size;		/* Non-zero if this option is a custom size */
   ppd_choice_t	**choices;		/* Choices */
   ppd_size_t	*size;			/* Custom page size */
   char		buf[1024];		/* Output buffer for feature */
@@ -311,9 +345,22 @@ ppdEmitFd(ppd_file_t    *ppd,		/* I - PPD file record */
       * Send DSC comments with option...
       */
 
-      snprintf(buf, sizeof(buf), "%%%%BeginFeature: *%s %s\n",
-               ((ppd_option_t *)choices[i]->option)->keyword,
-	       choices[i]->choice);
+      if ((strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageSize") == 0 ||
+           strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageRegion") == 0) &&
+          strcasecmp(choices[i]->choice, "Custom") == 0)
+      {
+        custom_size = 1;
+
+	strcpy(buf, "%%BeginFeature: *CustomPageSize True\n");
+      }
+      else
+      {
+        custom_size = 0;
+
+	snprintf(buf, sizeof(buf), "%%%%BeginFeature: *%s %s\n",
+        	 ((ppd_option_t *)choices[i]->option)->keyword,
+		 choices[i]->choice);
+      }
 
       if (write(fd, buf, strlen(buf)) < 1)
       {
@@ -321,19 +368,48 @@ ppdEmitFd(ppd_file_t    *ppd,		/* I - PPD file record */
         return (-1);
       }
 
-      if ((strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageSize") == 0 ||
-           strcasecmp(((ppd_option_t *)choices[i]->option)->keyword, "PageRegion") == 0) &&
-          strcasecmp(choices[i]->choice, "Custom") == 0)
+      if (custom_size)
       {
        /*
-        * Variable size; write out standard size options (this should
-	* eventually be changed to use the parameter positions defined
-	* in the PPD file...)
+        * Variable size; write out standard size options, using the
+	* parameter positions defined in the PPD file...
 	*/
 
+        ppd_attr_t	*attr;		/* PPD attribute */
+	int		pos,		/* Position of custom value */
+			values[5];	/* Values for custom command */
+
+
         size = ppdPageSize(ppd, "Custom");
-        snprintf(buf, sizeof(buf), "%.0f %.0f 0 0 0\n", size->width,
-	         size->length);
+
+        memset(values, 0, sizeof(values));
+
+	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize", "Width")) != NULL)
+	{
+	  pos = atoi(attr->value) - 1;
+
+          if (pos < 0 || pos > 4)
+	    pos = 0;
+	}
+	else
+	  pos = 0;
+
+	values[pos] = (int)size->width;
+
+	if ((attr = ppdFindAttr(ppd, "ParamCustomPageSize", "Height")) != NULL)
+	{
+	  pos = atoi(attr->value) - 1;
+
+          if (pos < 0 || pos > 4)
+	    pos = 1;
+	}
+	else
+	  pos = 1;
+
+	values[pos] = (int)size->length;
+
+        snprintf(buf, sizeof(buf), "%d %d %d %d %d\n", values[0], values[1],
+	         values[2], values[3], values[4]);
 
 	if (write(fd, buf, strlen(buf)) < 1)
 	{
@@ -485,8 +561,10 @@ static void
 ppd_handle_media(ppd_file_t *ppd)
 {
   ppd_choice_t	*manual_feed,		/* ManualFeed choice, if any */
-		*input_slot;		/* InputSlot choice, if any */
+		*input_slot,		/* InputSlot choice, if any */
+		*page;			/* PageSize/PageRegion */
   ppd_size_t	*size;			/* Current media size */
+  ppd_attr_t	*rpr;			/* RequiresPageRegion value */
 
 
  /*
@@ -502,6 +580,14 @@ ppd_handle_media(ppd_file_t *ppd)
   manual_feed = ppdFindMarkedChoice(ppd, "ManualFeed");
   input_slot  = ppdFindMarkedChoice(ppd, "InputSlot");
 
+  if (input_slot != NULL)
+    rpr = ppdFindAttr(ppd, "RequiresPageRegion", input_slot->choice);
+  else
+    rpr = NULL;
+
+  if (!rpr)
+    rpr = ppdFindAttr(ppd, "RequiresPageRegion", "All");
+
   if (strcasecmp(size->name, "Custom") == 0 ||
       (manual_feed == NULL && input_slot == NULL) ||
       (manual_feed != NULL && strcasecmp(manual_feed->choice, "False") == 0) ||
@@ -522,6 +608,22 @@ ppd_handle_media(ppd_file_t *ppd)
     */
 
     ppdMarkOption(ppd, "PageRegion", size->name);
+
+    if ((rpr && rpr->value && !strcmp(rpr->value, "False")) ||
+        (!rpr && !ppd->num_filters))
+    {
+     /*
+      * Either the PPD file specifies no PageRegion code or the PPD file
+      * not for a CUPS raster driver and thus defaults to no PageRegion
+      * code...  Unmark the PageRegion choice so that we don't output the
+      * code...
+      */
+
+      page = ppdFindMarkedChoice(ppd, "PageRegion");
+
+      if (page)
+        page->marked = 0;
+    }
   }
 }
 
@@ -544,5 +646,5 @@ ppd_sort(ppd_choice_t **c1,	/* I - First choice */
 
 
 /*
- * End of "$Id: emit.c,v 1.29 2002/12/17 18:56:41 swdev Exp $".
+ * End of "$Id: emit.c,v 1.32 2003/05/25 14:39:17 mike Exp $".
  */
