@@ -1,5 +1,5 @@
 /*
- * "$Id: encode.c,v 1.10 2002/12/17 18:56:41 swdev Exp $"
+ * "$Id: encode.c,v 1.14 2003/02/13 03:23:13 mike Exp $"
  *
  *   Option encoding routines for the Common UNIX Printing System (CUPS).
  *
@@ -49,7 +49,7 @@ typedef struct
   ipp_tag_t	value_tag;
 } ipp_option_t;
 
-static ipp_option_t	ipp_options[] =
+static const ipp_option_t ipp_options[] =
 			{
 			  { "blackplot",		IPP_TAG_BOOLEAN },
 			  { "brightness",		IPP_TAG_INTEGER },
@@ -59,8 +59,12 @@ static ipp_option_t	ipp_options[] =
 			  { "fitplot",			IPP_TAG_BOOLEAN },
 			  { "gamma",			IPP_TAG_INTEGER },
 			  { "hue",			IPP_TAG_INTEGER },
+			  { "job-k-limit",		IPP_TAG_INTEGER },
+			  { "job-page-limit",		IPP_TAG_INTEGER },
 			  { "job-priority",		IPP_TAG_INTEGER },
+			  { "job-quota-period",		IPP_TAG_INTEGER },
 			  { "landscape",		IPP_TAG_BOOLEAN },
+			  { "media",			IPP_TAG_KEYWORD },
 			  { "mirror",			IPP_TAG_BOOLEAN },
 			  { "natural-scaling",		IPP_TAG_INTEGER },
 			  { "number-up",		IPP_TAG_INTEGER },
@@ -77,6 +81,7 @@ static ipp_option_t	ipp_options[] =
 			  { "print-quality",		IPP_TAG_ENUM },
 			  { "saturation",		IPP_TAG_INTEGER },
 			  { "scaling",			IPP_TAG_INTEGER },
+			  { "sides",			IPP_TAG_KEYWORD },
 			  { "wrap",			IPP_TAG_BOOLEAN }
 			};
 
@@ -255,7 +260,7 @@ cupsEncodeOptions(ipp_t         *ipp,		/* I - Request to add to */
     * Scan the value string for values...
     */
 
-    for (j = 0; *val != '\0' || j == 0; val = sep, j ++)
+    for (j = 0; j < count; val = sep, j ++)
     {
      /*
       * Find the end of this value and mark it if needed...
@@ -377,5 +382,5 @@ cupsEncodeOptions(ipp_t         *ipp,		/* I - Request to add to */
 
 
 /*
- * End of "$Id: encode.c,v 1.10 2002/12/17 18:56:41 swdev Exp $".
+ * End of "$Id: encode.c,v 1.14 2003/02/13 03:23:13 mike Exp $".
  */
