@@ -1,5 +1,5 @@
 /*
- * "$Id: http.c,v 1.108 2002/12/17 18:56:42 swdev Exp $"
+ * "$Id: http.c,v 1.120 2003/05/27 14:03:37 mike Exp $"
  *
  *   HTTP routines for the Common UNIX Printing System (CUPS).
  *
@@ -29,6 +29,7 @@
  *                          default HTTP proxy (if any).
  *   httpCheck()          - Check to see if there is a pending response from
  *                          the server.
+ *   httpClearCookie()    - Clear the cookie value(s).
  *   httpClose()          - Close an HTTP connection...
  *   httpConnect()        - Connect to a HTTP server.
  *   httpConnectEncrypt() - Connect to a HTTP server using encryption.
@@ -45,6 +46,8 @@
  *   httpTrace()          - Send an TRACE request to the server.
  *   httpFlush()          - Flush data from a HTTP connection.
  *   httpRead()           - Read data from a HTTP connection.
+ *   httpSetCookie()      - Set the cookie value(s)...
+ *   httpWait()           - Wait for data available on a connection.
  *   httpWrite()          - Write data to a HTTP connection.
  *   httpGets()           - Get a line of text from a HTTP connection.
  *   httpPrintf()         - Print a formatted string to a HTTP connection.
@@ -59,6 +62,12 @@
  *   http_send()          - Send a request with all fields and the trailing
  *                          blank line.
  *   http_upgrade()       - Force upgrade to TLS encryption.
+ *   http_setup_ssl()     - Set up SSL/TLS on a connection.
+ *   http_shutdown_ssl()  - Shut down SSL/TLS on a connection.
+ *   http_read_ssl()      - Read from a SSL/TLS connection.
+ *   http_write_ssl()     - Write to a SSL/TLS connection.
+ *   CDSAReadFunc()       - Read function for CDSA decryption code.
+ *   CDSAWriteFunc()      - Write function for CDSA encryption code.
  */
 
 /*
@@ -74,18 +83,15 @@
 #include <errno.h>
 
 #include "http.h"
+#include "http-private.h"
 #include "debug.h"
 
 #ifndef WIN32
 #  include <signal.h>
+#  include <sys/time.h>
+#  include <sys/resource.h>
 #endif /* !WIN32 */
 
-#ifdef HAVE_LIBSSL
-#  include <openssl/err.h>
-#  include <openssl/rand.h>
-#  include <openssl/ssl.h>
-#endif /* HAVE_LIBSSL */
-
 
 /*
  * Some operating systems have done away with the Fxxxx constants for
@@ -104,16 +110,24 @@
 static http_field_t	http_field(const char *name);
 static int		http_send(http_t *http, http_state_t request,
 			          const char *uri);
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
 static int		http_upgrade(http_t *http);
-#endif /* HAVE_LIBSSL */
+static int		http_setup_ssl(http_t *http);
+static void		http_shutdown_ssl(http_t *http);
+static int		http_read_ssl(http_t *http, char *buf, int len);
+static int		http_write_ssl(http_t *http, const char *buf, int len);
+#  ifdef HAVE_CDSASSL
+static OSStatus		CDSAReadFunc(SSLConnectionRef connection, void *data, size_t *dataLength);
+static OSStatus		CDSAWriteFunc(SSLConnectionRef connection, const void *data, size_t *dataLength);
+#  endif /* HAVE_CDSASSL */
+#endif /* HAVE_SSL */
 
 
 /*
  * Local globals...
  */
 
-static const char	*http_fields[] =
+static const char * const http_fields[] =
 			{
 			  "Accept-Language",
 			  "Accept-Ranges",
@@ -143,7 +157,7 @@ static const char	*http_fields[] =
 			  "User-Agent",
 			  "WWW-Authenticate"
 			};
-static const char	*days[7] =
+static const char * const days[7] =
 			{
 			  "Sun",
 			  "Mon",
@@ -153,7 +167,7 @@ static const char	*days[7] =
 			  "Fri",
 			  "Sat"
 			};
-static const char	*months[12] =
+static const char * const months[12] =
 			{
 			  "Jan",
 			  "Feb",
@@ -210,6 +224,10 @@ httpInitialize(void)
   signal(SIGPIPE, SIG_IGN);
 #endif /* WIN32 */
 
+#ifdef HAVE_GNUTLS
+  gnutls_global_init();
+#endif /* HAVE_GNUTLS */
+
 #ifdef HAVE_LIBSSL
   SSL_load_error_strings();
   SSL_library_init();
@@ -240,31 +258,25 @@ httpInitialize(void)
 int				/* O - 0 = no data, 1 = data available */
 httpCheck(http_t *http)		/* I - HTTP connection */
 {
-  fd_set	input;		/* Input set for select() */
-  struct timeval timeout;	/* Timeout */
+  return (httpWait(http, 0));
+}
 
 
- /*
-  * First see if there is data in the buffer...
-  */
+/*
+ * 'httpClearCookie()' - Clear the cookie value(s).
+ */
 
-  if (http == NULL)
-    return (0);
+void
+httpClearCookie(http_t *http)			/* I - Connection */
+{
+  if (!http)
+    return;
 
-  if (http->used)
-    return (1);
-
- /*
-  * Then try doing a select() to poll the socket...
-  */
-
-  FD_ZERO(&input);
-  FD_SET(http->fd, &input);
-
-  timeout.tv_sec  = 0;
-  timeout.tv_usec = 0;
-
-  return (select(http->fd + 1, &input, NULL, NULL, &timeout) > 0);
+  if (http->cookie)
+  {
+    free(http->cookie);
+    http->cookie = NULL;
+  }
 }
 
 
@@ -275,28 +287,19 @@ httpCheck(http_t *http)		/* I - HTTP connection */
 void
 httpClose(http_t *http)		/* I - Connection to close */
 {
-#ifdef HAVE_LIBSSL
-  SSL_CTX	*context;	/* Context for encryption */
-  SSL		*conn;		/* Connection for encryption */
-#endif /* HAVE_LIBSSL */
-
-
   if (!http)
     return;
 
-#ifdef HAVE_LIBSSL
+  if (http->input_set)
+    free(http->input_set);
+
+  if (http->cookie)
+    free(http->cookie);
+
+#ifdef HAVE_SSL
   if (http->tls)
-  {
-    conn    = (SSL *)(http->tls);
-    context = SSL_get_SSL_CTX(conn);
-
-    SSL_shutdown(conn);
-    SSL_CTX_free(context);
-    SSL_free(conn);
-
-    http->tls = NULL;
-  }
-#endif /* HAVE_LIBSSL */
+    http_shutdown_ssl(http);
+#endif /* HAVE_SSL */
 
 #ifdef WIN32
   closesocket(http->fd);
@@ -450,7 +453,7 @@ int					/* O - -1 on error, 0 on success */
 httpEncryption(http_t            *http,	/* I - HTTP data */
                http_encryption_t e)	/* I - New encryption preference */
 {
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
   if (!http)
     return (0);
 
@@ -468,7 +471,7 @@ httpEncryption(http_t            *http,	/* I - HTTP data */
     return (-1);
   else
     return (0);
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 }
 
 
@@ -480,23 +483,11 @@ int				/* O - 0 on success, non-zero on failure */
 httpReconnect(http_t *http)	/* I - HTTP data */
 {
   int		val;		/* Socket option value */
-#ifdef HAVE_LIBSSL
-  SSL_CTX	*context;	/* Context for encryption */
-  SSL		*conn;		/* Connection for encryption */
-
 
+#ifdef HAVE_SSL
   if (http->tls)
-  {
-    conn    = (SSL *)(http->tls);
-    context = SSL_get_SSL_CTX(conn);
-
-    SSL_shutdown(conn);
-    SSL_CTX_free(context);
-    SSL_free(conn);
-
-    http->tls = NULL;
-  }
-#endif /* HAVE_LIBSSL */
+    http_shutdown_ssl(http);
+#endif /* HAVE_SSL */
 
  /*
   * Close any previously open socket...
@@ -575,43 +566,27 @@ httpReconnect(http_t *http)	/* I - HTTP data */
   http->error  = 0;
   http->status = HTTP_CONTINUE;
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
   if (http->encryption == HTTP_ENCRYPT_ALWAYS)
   {
    /*
     * Always do encryption via SSL.
     */
 
-    context = SSL_CTX_new(SSLv23_method());
-    conn    = SSL_new(context);
-
-    SSL_set_fd(conn, http->fd);
-    if (SSL_connect(conn) != 1)
+    if (http_setup_ssl(http) != 0)
     {
-      SSL_CTX_free(context);
-      SSL_free(conn);
-
-#ifdef WIN32
-      http->error  = WSAGetLastError();
-#else
-      http->error  = errno;
-#endif /* WIN32 */
-      http->status = HTTP_ERROR;
-
 #ifdef WIN32
       closesocket(http->fd);
 #else
       close(http->fd);
-#endif
+#endif /* WIN32 */
 
       return (-1);
     }
-
-    http->tls = conn;
   }
   else if (http->encryption == HTTP_ENCRYPT_REQUIRED)
     return (http_upgrade(http));
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 
   return (0);
 }
@@ -857,7 +832,10 @@ httpFlush(http_t *http)	/* I - HTTP data */
   char	buffer[8192];	/* Junk buffer */
 
 
-  while (httpRead(http, buffer, sizeof(buffer)) > 0);
+  if (http->state != HTTP_WAITING)
+  {
+    while (httpRead(http, buffer, sizeof(buffer)) > 0);
+  }
 }
 
 
@@ -931,16 +909,19 @@ httpRead(http_t *http,			/* I - HTTP data */
     * Buffer small reads for better performance...
     */
 
+    if (!http->blocking && !httpWait(http, 1000))
+      return (0);
+
     if (http->data_remaining > sizeof(http->buffer))
       bytes = sizeof(http->buffer);
     else
       bytes = http->data_remaining;
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
     if (http->tls)
-      bytes = SSL_read((SSL *)(http->tls), http->buffer, bytes);
+      bytes = http_read_ssl(http, http->buffer, bytes);
     else
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
     {
       DEBUG_printf(("httpRead: reading %d bytes from socket into buffer...\n",
                     bytes));
@@ -967,7 +948,10 @@ httpRead(http_t *http,			/* I - HTTP data */
 #endif /* WIN32 */
     }
     else
+    {
+      http->error = EPIPE;
       return (0);
+    }
   }
 
   if (http->used > 0)
@@ -985,12 +969,20 @@ httpRead(http_t *http,			/* I - HTTP data */
     if (http->used > 0)
       memmove(http->buffer, http->buffer + length, http->used);
   }
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
   else if (http->tls)
-    bytes = SSL_read((SSL *)(http->tls), buffer, length);
-#endif /* HAVE_LIBSSL */
+  {
+    if (!http->blocking && !httpWait(http, 1000))
+      return (0);
+
+    bytes = http_read_ssl(http, buffer, length);
+  }
+#endif /* HAVE_SSL */
   else
   {
+    if (!http->blocking && !httpWait(http, 1000))
+      return (0);
+
     DEBUG_printf(("httpRead: reading %d bytes from socket...\n", length));
     bytes = recv(http->fd, buffer, length, 0);
     DEBUG_printf(("httpRead: read %d bytes from socket...\n", bytes));
@@ -1009,6 +1001,11 @@ httpRead(http_t *http,			/* I - HTTP data */
       http->error = errno;
 #endif /* WIN32 */
   }
+  else
+  {
+    http->error = EPIPE;
+    return (0);
+  }
 
   if (http->data_remaining == 0)
   {
@@ -1060,6 +1057,117 @@ httpRead(http_t *http,			/* I - HTTP data */
 }
 
 
+/*
+ * 'httpSetCookie()' - Set the cookie value(s)...
+ */
+
+void
+httpSetCookie(http_t     *http,			/* I - Connection */
+              const char *cookie)		/* I - Cookie string */
+{
+  if (!http)
+    return;
+
+  if (http->cookie)
+    free(http->cookie);
+
+  if (cookie)
+    http->cookie = strdup(cookie);
+  else
+    http->cookie = NULL;
+}
+
+
+/*
+ * 'httpWait()' - Wait for data available on a connection.
+ */
+
+int					/* O - 1 if data is available, 0 otherwise */
+httpWait(http_t *http,			/* I - HTTP data */
+         int    msec)			/* I - Milliseconds to wait */
+{
+#ifndef WIN32
+  struct rlimit		limit;          /* Runtime limit */
+#endif /* !WIN32 */
+  struct timeval	timeout;	/* Timeout */
+  int			nfds;		/* Result from select() */
+
+
+ /*
+  * First see if there is data in the buffer...
+  */
+
+  if (http == NULL)
+    return (0);
+
+  if (http->used)
+    return (1);
+
+#ifdef HAVE_SSL
+  if (http->tls)
+  {
+#  ifdef HAVE_LIBSSL
+    if (SSL_pending((SSL *)(http->tls)))
+      return (1);
+#  elif defined(HAVE_GNUTLS)
+    if (gnutls_record_check_pending(((http_tls_t *)(http->tls))->session))
+      return (1);
+#  elif defined(HAVE_CDSASSL)
+    size_t bytes;			/* Bytes that are available */
+
+    if (!SSLGetBufferedReadSize((SSLContextRef)http->tls, &bytes) && bytes > 0)
+      return;
+#  endif /* HAVE_LIBSSL */
+  }
+#endif /* HAVE_SSL */
+
+ /*
+  * Then try doing a select() to poll the socket...
+  */
+
+  if (!http->input_set)
+  {
+#ifdef WIN32
+   /*
+    * Windows has a fixed-size select() structure, different (surprise,
+    * surprise!) from all UNIX implementations.  Just allocate this
+    * fixed structure...
+    */
+
+    http->input_set = calloc(1, sizeof(fd_set));
+#else
+   /*
+    * Allocate the select() input set based upon the max number of file
+    * descriptors available for this process...
+    */
+
+    getrlimit(RLIMIT_NOFILE, &limit);
+
+    http->input_set = calloc(1, (limit.rlim_cur + 7) / 8);
+#endif /* WIN32 */
+
+    if (!http->input_set)
+      return (0);
+  }
+
+  FD_SET(http->fd, http->input_set);
+
+  if (msec >= 0)
+  {
+    timeout.tv_sec  = msec / 1000;
+    timeout.tv_usec = (msec % 1000) * 1000;
+
+    nfds = select(http->fd + 1, http->input_set, NULL, NULL, &timeout);
+  }
+  else
+    nfds = select(http->fd + 1, http->input_set, NULL, NULL, NULL);
+
+  FD_CLR(http->fd, http->input_set);
+
+  return (nfds > 0);
+}
+
+
 /*
  * 'httpWrite()' - Write data to a HTTP connection.
  */
@@ -1110,11 +1218,11 @@ httpWrite(http_t     *http,		/* I - HTTP data */
 
   while (length > 0)
   {
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
     if (http->tls)
-      bytes = SSL_write((SSL *)(http->tls), buffer, length);
+      bytes = http_write_ssl(http, buffer, length);
     else
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
     bytes = send(http->fd, buffer, length, 0);
 
     if (bytes < 0)
@@ -1247,13 +1355,15 @@ httpGets(char   *line,			/* I - Line to read into */
       * No newline; see if there is more data to be read...
       */
 
-#ifdef HAVE_LIBSSL
+      if (!http->blocking && !httpWait(http, 1000))
+        return (NULL);
+
+#ifdef HAVE_SSL
       if (http->tls)
-        bytes = SSL_read((SSL *)(http->tls), bufend,
-	                 HTTP_MAX_BUFFER - http->used);
+	bytes = http_read_ssl(http, bufend, HTTP_MAX_BUFFER - http->used);
       else
-#endif /* HAVE_LIBSSL */
-      bytes = recv(http->fd, bufend, HTTP_MAX_BUFFER - http->used, 0);
+#endif /* HAVE_SSL */
+        bytes = recv(http->fd, bufend, HTTP_MAX_BUFFER - http->used, 0);
 
       if (bytes < 0)
       {
@@ -1285,8 +1395,7 @@ httpGets(char   *line,			/* I - Line to read into */
       }
       else if (bytes == 0)
       {
-        if (http->blocking)
-	  http->error = EPIPE;
+	http->error = EPIPE;
 
         return (NULL);
       }
@@ -1371,11 +1480,11 @@ httpPrintf(http_t     *http,		/* I - HTTP data */
 
   for (tbytes = 0, bufptr = buf; tbytes < bytes; tbytes += nbytes, bufptr += nbytes)
   {
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
     if (http->tls)
-      nbytes = SSL_write((SSL *)(http->tls), bufptr, bytes - tbytes);
+      nbytes = http_write_ssl(http, bufptr, bytes - tbytes);
     else
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
     nbytes = send(http->fd, bufptr, bytes - tbytes, 0);
 
     if (nbytes < 0)
@@ -1474,10 +1583,6 @@ httpUpdate(http_t *http)		/* I - HTTP data */
   http_field_t	field;			/* Field index */
   int		major, minor;		/* HTTP version numbers */
   http_status_t	status;			/* Authorization status */
-#ifdef HAVE_LIBSSL
-  SSL_CTX	*context;		/* Context for encryption */
-  SSL		*conn;			/* Connection for encryption */
-#endif /* HAVE_LIBSSL */
 
 
   DEBUG_printf(("httpUpdate(%p)\n", http));
@@ -1511,25 +1616,11 @@ httpUpdate(http_t *http)		/* I - HTTP data */
       if (http->status == HTTP_CONTINUE)
         return (http->status);
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
       if (http->status == HTTP_SWITCHING_PROTOCOLS && !http->tls)
       {
-	context = SSL_CTX_new(SSLv23_method());
-	conn    = SSL_new(context);
-
-	SSL_set_fd(conn, http->fd);
-	if (SSL_connect(conn) != 1)
+	if (http_setup_ssl(http) != 0)
 	{
-	  SSL_CTX_free(context);
-	  SSL_free(conn);
-
-#ifdef WIN32
-	  http->error  = WSAGetLastError();
-#else
-	  http->error  = errno;
-#endif /* WIN32 */
-	  http->status = HTTP_ERROR;
-
 #ifdef WIN32
 	  closesocket(http->fd);
 #else
@@ -1539,11 +1630,9 @@ httpUpdate(http_t *http)		/* I - HTTP data */
 	  return (HTTP_ERROR);
 	}
 
-	http->tls = conn;
-
         return (HTTP_CONTINUE);
       }
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 
       httpGetLength(http);
 
@@ -1554,6 +1643,7 @@ httpUpdate(http_t *http)		/* I - HTTP data */
 	case HTTP_POST_RECV :
 	case HTTP_PUT :
 	    http->state ++;
+	case HTTP_POST_SEND :
 	    break;
 
 	default :
@@ -1589,13 +1679,29 @@ httpUpdate(http_t *http)		/* I - HTTP data */
       * Be tolerants of servers that send unknown attribute fields...
       */
 
-      if ((field = http_field(line)) == HTTP_FIELD_UNKNOWN)
+      if (!strcasecmp(line, "expect"))
+      {
+       /*
+        * "Expect: 100-continue" or similar...
+	*/
+
+        http->expect = (http_status_t)atoi(value);
+      }
+      else if (!strcasecmp(line, "cookie"))
+      {
+       /*
+        * "Cookie: name=value[; name=value ...]" - replaces previous cookies...
+	*/
+
+        httpSetCookie(http, value);
+      }
+      else if ((field = http_field(line)) == HTTP_FIELD_UNKNOWN)
       {
         DEBUG_printf(("httpUpdate: unknown field %s seen!\n", line));
         continue;
       }
-
-      httpSetField(http, field, value);
+      else
+        httpSetField(http, field, value);
     }
     else
     {
@@ -1702,7 +1808,7 @@ httpEncode64(char       *out,	/* I - String to write to */
              const char *in)	/* I - String to read from */
 {
   char		*outptr;	/* Output pointer */
-  static char	base64[] =	/* Base64 characters... */
+  static const char base64[] =	/* Base64 characters... */
   		{
 		  "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
 		  "abcdefghijklmnopqrstuvwxyz"
@@ -1755,7 +1861,7 @@ httpEncode64(char       *out,	/* I - String to write to */
 int				/* O - Content length */
 httpGetLength(http_t *http)	/* I - HTTP data */
 {
-  DEBUG_printf(("httpGetLength(%p)\n", http));
+  DEBUG_printf(("httpGetLength(%p), state = %d\n", http, http->state));
 
   if (strcasecmp(http->fields[HTTP_FIELD_TRANSFER_ENCODING], "chunked") == 0)
   {
@@ -1818,8 +1924,8 @@ http_send(http_t       *http,	/* I - HTTP data */
   int		i;		/* Looping var */
   char		*ptr,		/* Pointer in buffer */
 		buf[1024];	/* Encoded URI buffer */
-  static const char *codes[] =	/* Request code strings */
-		{
+  static const char * const codes[] =
+		{		/* Request code strings */
 		  NULL,
 		  "OPTIONS",
 		  "GET",
@@ -1834,7 +1940,7 @@ http_send(http_t       *http,	/* I - HTTP data */
 		  "TRACE",
 		  "CLOSE"
 		};
-  static const char *hex = "0123456789ABCDEF";
+  static const char hex[] = "0123456789ABCDEF";
 				/* Hex digits */
 
 
@@ -1877,13 +1983,13 @@ http_send(http_t       *http,	/* I - HTTP data */
 
   http->status = HTTP_CONTINUE;
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
   if (http->encryption == HTTP_ENCRYPT_REQUIRED && !http->tls)
   {
     httpSetField(http, HTTP_FIELD_CONNECTION, "Upgrade");
     httpSetField(http, HTTP_FIELD_UPGRADE, "TLS/1.0,SSL/2.0,SSL/3.0");
   }
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 
   if (httpPrintf(http, "%s %s HTTP/1.1\r\n", codes[request], buf) < 1)
   {
@@ -1915,7 +2021,7 @@ http_send(http_t       *http,	/* I - HTTP data */
 }
 
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
 /*
  * 'http_upgrade()' - Force upgrade to TLS encryption.
  */
@@ -1991,11 +2097,11 @@ http_upgrade(http_t *http)	/* I - HTTP data */
 
     DEBUG_puts("Server does not support HTTP upgrade!");
 
-#ifdef WIN32
+#  ifdef WIN32
     closesocket(http->fd);
-#else
+#  else
     close(http->fd);
-#endif
+#  endif
 
     http->fd = -1;
 
@@ -2004,9 +2110,282 @@ http_upgrade(http_t *http)	/* I - HTTP data */
   else
     return (ret);
 }
-#endif /* HAVE_LIBSSL */
 
 
 /*
- * End of "$Id: http.c,v 1.108 2002/12/17 18:56:42 swdev Exp $".
+ * 'http_setup_ssl()' - Set up SSL/TLS support on a connection.
+ */
+
+static int				/* O - Status of connection */
+http_setup_ssl(http_t *http)		/* I - HTTP data */
+{
+#  ifdef HAVE_LIBSSL
+  SSL_CTX	*context;	/* Context for encryption */
+  SSL		*conn;		/* Connection for encryption */
+#  elif defined(HAVE_GNUTLS)
+  http_tls_t	*conn;		/* TLS session object */
+  gnutls_certificate_client_credentials *credentials;
+				/* TLS credentials */
+#  elif defined(HAVE_CDSASSL)
+  SSLContextRef	conn;		/* Context for encryption */
+  OSStatus	error;		/* Error info */
+#  endif /* HAVE_LIBSSL */
+
+
+#  ifdef HAVE_LIBSSL
+  context = SSL_CTX_new(SSLv23_client_method());
+  conn    = SSL_new(context);
+
+  SSL_set_fd(conn, http->fd);
+  if (SSL_connect(conn) != 1)
+  {
+    SSL_CTX_free(context);
+    SSL_free(conn);
+
+#    ifdef WIN32
+    http->error  = WSAGetLastError();
+#    else
+    http->error  = errno;
+#    endif /* WIN32 */
+    http->status = HTTP_ERROR;
+
+    return (HTTP_ERROR);
+  }
+
+#  elif defined(HAVE_GNUTLS)
+  conn = (http_tls_t *)malloc(sizeof(http_tls_t));
+
+  if (conn == NULL)
+  {
+    http->error  = errno;
+    http->status = HTTP_ERROR;
+
+    return (-1);
+  }
+
+  credentials = (gnutls_certificate_client_credentials *)
+                    malloc(sizeof(gnutls_certificate_client_credentials));
+  if (credentials == NULL)
+  {
+    free(conn);
+
+    http->error = errno;
+    http->status = HTTP_ERROR;
+
+    return (-1);
+  }
+
+  gnutls_certificate_allocate_credentials(credentials);
+
+  gnutls_init(&(conn->session), GNUTLS_CLIENT);
+  gnutls_set_default_priority(conn->session);
+  gnutls_credentials_set(conn->session, GNUTLS_CRD_CERTIFICATE, *credentials);
+  gnutls_transport_set_ptr(conn->session, http->fd);
+
+  if ((gnutls_handshake(conn->session)) != GNUTLS_E_SUCCESS)
+  {
+    http->error  = errno;
+    http->status = HTTP_ERROR;
+
+    return (-1);
+  }
+
+  conn->credentials = credentials;
+
+#  elif defined(HAVE_CDSASSL)
+  error = SSLNewContext(false, &conn);
+
+  if (!error)
+    error = SSLSetIOFuncs(conn, CDSAReadFunc, CDSAWriteFunc);
+
+  if (!error)
+    error = SSLSetConnection(conn, (SSLConnectionRef)http->fd);
+
+  if (!error)
+    error = SSLSetAllowsExpiredCerts(conn, true);
+
+  if (!error)
+    error = SSLSetAllowsAnyRoot(conn, true);
+
+  if (!error)
+    error = SSLHandshake(conn);
+
+  if (error != 0)
+  {
+    http->error  = error;
+    http->status = HTTP_ERROR;
+
+    SSLDisposeContext(conn);
+
+    close(http->fd);
+
+    return (-1);
+  }
+#  endif /* HAVE_CDSASSL */
+
+  http->tls = conn;
+  return (0);
+}
+
+
+/*
+ * 'http_shutdown_ssl()' - Shut down SSL/TLS on a connection.
+ */
+
+static void
+http_shutdown_ssl(http_t *http)	/* I - HTTP data */
+{
+#  ifdef HAVE_LIBSSL
+  SSL_CTX	*context;	/* Context for encryption */
+  SSL		*conn;		/* Connection for encryption */
+
+
+  conn    = (SSL *)(http->tls);
+  context = SSL_get_SSL_CTX(conn);
+
+  SSL_shutdown(conn);
+  SSL_CTX_free(context);
+  SSL_free(conn);
+
+#  elif defined(HAVE_GNUTLS)
+  http_tls_t      *conn;	/* Encryption session */
+  gnutls_certificate_client_credentials *credentials;
+				/* TLS credentials */
+
+
+  conn = (http_tls_t *)(http->tls);
+  credentials = (gnutls_certificate_client_credentials *)(conn->credentials);
+
+  gnutls_bye(conn->session, GNUTLS_SHUT_RDWR);
+  gnutls_deinit(conn->session);
+  gnutls_certificate_free_credentials(*credentials);
+  free(credentials);
+  free(conn);
+
+#  elif defined(HAVE_CDSASSL)
+  SSLClose((SSLContextRef)http->tls);
+  SSLDisposeContext((SSLContextRef)http->tls);
+#  endif /* HAVE_LIBSSL */
+
+  http->tls = NULL;
+}
+
+
+/*
+ * 'http_read_ssl()' - Read from a SSL/TLS connection.
+ */
+
+static int				/* O - Bytes read */
+http_read_ssl(http_t *http,		/* I - HTTP data */
+	      char   *buf,		/* I - Buffer to store data */
+	      int    len)		/* I - Length of buffer */
+{
+#  if defined(HAVE_LIBSSL)
+  return (SSL_read((SSL *)(http->tls), buf, len));
+
+#  elif defined(HAVE_GNUTLS)
+  return (gnutls_record_recv(((http_tls_t *)(http->tls))->session, buf, len));
+
+#  elif defined(HAVE_CDSASSL)
+  OSStatus	error;			/* Error info */
+  size_t	processed;		/* Number of bytes processed */
+
+
+  error = SSLRead((SSLContextRef)http->tls, buf, len, &processed);
+
+  if (error == 0)
+    return (processed);
+  else
+  {
+    http->error = error;
+
+    return (-1);
+  }
+#  endif /* HAVE_LIBSSL */
+}
+
+
+/*
+ * 'http_write_ssl()' - Write to a SSL/TLS connection.
+ */
+
+static int				/* O - Bytes written */
+http_write_ssl(http_t     *http,	/* I - HTTP data */
+	       const char *buf,		/* I - Buffer holding data */
+	       int        len)		/* I - Length of buffer */
+{
+#  if defined(HAVE_LIBSSL)
+  return (SSL_write((SSL *)(http->tls), buf, len));
+
+#  elif defined(HAVE_GNUTLS)
+  return (gnutls_record_send(((http_tls_t *)(http->tls))->session, buf, len));
+#  elif defined(HAVE_CDSASSL)
+  OSStatus	error;			/* Error info */
+  size_t	processed;		/* Number of bytes processed */
+
+
+  error = SSLWrite((SSLContextRef)http->tls, buf, len, &processed);
+
+  if (error == 0)
+    return (processed);
+  else
+  {
+    http->error = error;
+    return (-1);
+  }
+#  endif /* HAVE_LIBSSL */
+}
+
+
+#  if defined(HAVE_CDSASSL)
+/*
+ * 'CDSAReadFunc()' - Read function for CDSA decryption code.
+ */
+
+static OSStatus					/* O  - -1 on error, 0 on success */
+CDSAReadFunc(SSLConnectionRef connection,	/* I  - SSL/TLS connection */
+             void             *data,		/* I  - Data buffer */
+	     size_t           *dataLength)	/* IO - Number of bytes */
+{
+  ssize_t	bytes;				/* Number of bytes read */
+
+
+  bytes = recv((int)connection, data, *dataLength, 0);
+  if (bytes >= 0)
+  {
+    *dataLength = bytes;
+    return (0);
+  }
+  else
+    return (-1);
+}
+
+
+/*
+ * 'CDSAWriteFunc()' - Write function for CDSA encryption code.
+ */
+
+static OSStatus					/* O  - -1 on error, 0 on success */
+CDSAWriteFunc(SSLConnectionRef connection,	/* I  - SSL/TLS connection */
+              const void       *data,		/* I  - Data buffer */
+	      size_t           *dataLength)	/* IO - Number of bytes */
+{
+  ssize_t bytes;
+
+
+  bytes = write((int)connection, data, *dataLength);
+  if (bytes >= 0)
+  {
+    *dataLength = bytes;
+    return (0);
+  }
+  else
+    return (-1);
+}
+#  endif /* HAVE_CDSASSL */
+#endif /* HAVE_SSL */
+
+
+/*
+ * End of "$Id: http.c,v 1.120 2003/05/27 14:03:37 mike Exp $".
  */
