@@ -1,9 +1,9 @@
 /*
- * "$Id: string.h,v 1.13 2002/05/16 13:44:54 mike Exp $"
+ * "$Id: string.h,v 1.15 2002/12/17 18:56:43 swdev Exp $"
  *
  *   String definitions for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -45,6 +45,10 @@
 #    include <strings.h>
 #  endif /* HAVE_STRINGS_H */
 
+#  ifdef HAVE_BSTRING_H
+#    include <bstring.h>
+#  endif /* HAVE_BSTRING_H */
+
 
 /*
  * Stuff for WIN32 and OS/2...
@@ -120,5 +124,5 @@ extern int	cups_vsnprintf(char *, size_t, const char *, va_list);
 #endif /* !_CUPS_STRING_H_ */
 
 /*
- * End of "$Id: string.h,v 1.13 2002/05/16 13:44:54 mike Exp $".
+ * End of "$Id: string.h,v 1.15 2002/12/17 18:56:43 swdev Exp $".
  */
