@@ -1,5 +1,5 @@
 /*
- * "$Id: util.c,v 1.103 2002/12/19 15:08:46 mike Exp $"
+ * "$Id: util.c,v 1.106 2003/04/08 03:45:16 mike Exp $"
  *
  *   Printing utilities for the Common UNIX Printing System (CUPS).
  *
@@ -103,6 +103,7 @@ cupsCancelJob(const char *name,	/* I - Name of printer or class */
 
   if (!cups_connect(name, printer, hostname))
   {
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (0);
   }
@@ -186,6 +187,10 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
   int		digest_tries;		/* Number of tries with Digest */
 
 
+  DEBUG_printf(("cupsDoFileRequest(%p, %p, \'%s\', \'%s\')\n",
+                http, request, resource ? resource : "(null)",
+		filename ? filename : "(null)"));
+
   if (http == NULL || request == NULL || resource == NULL)
   {
     if (request != NULL)
@@ -195,9 +200,6 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
     return (NULL);
   }
 
-  DEBUG_printf(("cupsDoFileRequest(%p, %p, \'%s\', \'%s\')\n",
-                http, request, resource, filename ? filename : "(null)"));
-
  /*
   * See if we have a file to send...
   */
@@ -410,7 +412,7 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
       else
         break;
     }
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
     else if (status == HTTP_UPGRADE_REQUIRED)
     {
      /*
@@ -431,7 +433,7 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
 
       continue;
     }
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
     else if (status != HTTP_OK)
     {
       DEBUG_printf(("cupsDoFileRequest: error %d...\n", status));
@@ -459,6 +461,7 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
         * Delete the response...
 	*/
 
+        DEBUG_puts("IPP read error!");
 	ippDelete(response);
 	response = NULL;
 
@@ -494,7 +497,11 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
   else if (status == HTTP_UNAUTHORIZED)
     last_error = IPP_NOT_AUTHORIZED;
   else if (status != HTTP_OK)
+  {
+    DEBUG_printf(("HTTP error %d mapped to IPP_SERVICE_UNAVAILABLE!\n",
+                  status));
     last_error = IPP_SERVICE_UNAVAILABLE;
+  }
 
   return (response);
 }
@@ -553,6 +560,7 @@ cupsGetClasses(char ***classes)	/* O - Classes */
 
   if (!cups_connect("default", NULL, NULL))
   {
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (0);
   }
@@ -667,6 +675,7 @@ cupsGetDefault(void)
 
   if (!cups_connect("default", NULL, NULL))
   {
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (NULL);
   }
@@ -744,7 +753,7 @@ cupsGetJobs(cups_job_t **jobs,		/* O - Job data */
 		*title,			/* job-name */
 		*user;			/* job-originating-user-name */
   char		uri[HTTP_MAX_URI];	/* URI for jobs */
-  static const char *attrs[] =		/* Requested attributes */
+  static const char * const attrs[] =	/* Requested attributes */
 		{
 		  "job-id",
 		  "job-priority",
@@ -772,6 +781,7 @@ cupsGetJobs(cups_job_t **jobs,		/* O - Job data */
 
   if (!cups_connect("default", NULL, NULL))
   {
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (0);
   }
@@ -1003,8 +1013,8 @@ cupsGetPPD(const char *name)		/* I - Printer name */
   char		prompt[1024];		/* Prompt string */
   int		digest_tries;		/* Number of tries with Digest */
   static char	filename[HTTP_MAX_URI];	/* Local filename */
-  static const char *requested_attrs[] =/* Requested attributes */
-		{
+  static const char * const requested_attrs[] =
+		{			/* Requested attributes */
 		  "printer-uri-supported",
 		  "printer-type",
 		  "member-uris"
@@ -1023,6 +1033,7 @@ cupsGetPPD(const char *name)		/* I - Printer name */
 
   if (!cups_connect(name, printer, hostname))
   {
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (NULL);
   }
@@ -1133,6 +1144,7 @@ cupsGetPPD(const char *name)		/* I - Printer name */
     if ((cups_server = httpConnectEncrypt(hostname, ippPort(),
                                           cupsEncryption())) == NULL)
     {
+      DEBUG_puts("Unable to connect to server!");
       last_error = IPP_SERVICE_UNAVAILABLE;
       return (NULL);
     }
@@ -1260,7 +1272,7 @@ cupsGetPPD(const char *name)		/* I - Printer name */
 
       continue;
     }
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
     else if (status == HTTP_UPGRADE_REQUIRED)
     {
      /*
@@ -1281,7 +1293,7 @@ cupsGetPPD(const char *name)		/* I - Printer name */
 
       continue;
     }
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
   }
   while (status == HTTP_UNAUTHORIZED || status == HTTP_UPGRADE_REQUIRED);
 
@@ -1297,6 +1309,7 @@ cupsGetPPD(const char *name)		/* I - Printer name */
           last_error = IPP_NOT_FOUND;
 	  break;
       case HTTP_ERROR :
+          DEBUG_puts("Mapping HTTP error to IPP_SERVICE_UNAVAILABLE");
           last_error = IPP_SERVICE_UNAVAILABLE;
 	  break;
       case HTTP_UNAUTHORIZED :
@@ -1354,6 +1367,7 @@ cupsGetPrinters(char ***printers)	/* O - Printers */
 
   if (!cups_connect("default", NULL, NULL))
   {
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (0);
   }
@@ -1502,6 +1516,7 @@ cupsPrintFiles(const char    *name,	/* I - Printer or class name */
   {
     DEBUG_printf(("cupsPrintFile: Unable to open connection - %s.\n",
                   strerror(errno)));
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (0);
   }
@@ -1695,6 +1710,7 @@ cups_connect(const char *name,		/* I - Destination (printer[@host]) */
   if ((cups_server = httpConnectEncrypt(hostname, ippPort(),
                                         cupsEncryption())) == NULL)
   {
+    DEBUG_puts("Unable to connect to server!");
     last_error = IPP_SERVICE_UNAVAILABLE;
     return (NULL);
   }
@@ -1787,5 +1803,5 @@ cups_local_auth(http_t *http)	/* I - Connection */
 
 
 /*
- * End of "$Id: util.c,v 1.103 2002/12/19 15:08:46 mike Exp $".
+ * End of "$Id: util.c,v 1.106 2003/04/08 03:45:16 mike Exp $".
  */
