@@ -1,9 +1,9 @@
 /*
- * "$Id: image-pnm.c,v 1.10 2002/04/19 16:17:27 mike Exp $"
+ * "$Id: image-pnm.c,v 1.12 2002/12/17 18:59:27 swdev Exp $"
  *
  *   Portable Any Map file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -132,6 +132,22 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
   else
     maxval = 1;
 
+  if (img->xsize == 0 || img->xsize > IMAGE_MAX_WIDTH ||
+      img->ysize == 0 || img->ysize > IMAGE_MAX_HEIGHT)
+  {
+    fprintf(stderr, "ERROR: Bad PNM dimensions %dx%d!\n",
+            img->xsize, img->ysize);
+    fclose(fp);
+    return (1);
+  }
+
+  if (maxval == 0)
+  {
+    fprintf(stderr, "ERROR: Bad PNM max value %d!\n", maxval);
+    fclose(fp);
+    return (1);
+  }
+
   if (format == 1 || format == 2 || format == 4 || format == 5)
     img->colorspace = secondary;
   else
@@ -286,5 +302,5 @@ ImageReadPNM(image_t    *img,		/* IO - Image */
 
 
 /*
- * End of "$Id: image-pnm.c,v 1.10 2002/04/19 16:17:27 mike Exp $".
+ * End of "$Id: image-pnm.c,v 1.12 2002/12/17 18:59:27 swdev Exp $".
  */
