@@ -1,9 +1,9 @@
 /*
- * "$Id: image-tiff.c,v 1.23 2002/04/19 16:17:27 mike Exp $"
+ * "$Id: image-tiff.c,v 1.25 2002/12/17 18:59:27 swdev Exp $"
  *
  *   TIFF file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -173,6 +173,12 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
       img->yppi = 128;
     }
 
+    if (img->xppi == 0 || img->yppi == 0)
+    {
+      fputs("ERROR: Bad TIFF resolution.\n", stderr);
+      img->xppi = img->yppi = 128;
+    }
+
     fprintf(stderr, "DEBUG: TIFF resolution = %fx%f, units=%d\n",
             xres, yres, resunit);
     fprintf(stderr, "DEBUG: Stored resolution = %dx%d PPI\n",
@@ -188,6 +194,23 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
   else
     alpha = 0;
 
+ /*
+  * Check the size of the image...
+  */
+
+  if (width == 0 || width > IMAGE_MAX_WIDTH ||
+      height == 0 || height > IMAGE_MAX_HEIGHT ||
+      (bits != 1 && bits != 2 && bits != 4 && bits != 8) ||
+      samples < 1 || samples > 4)
+  {
+    fprintf(stderr, "ERROR: Bad TIFF dimensions %ux%ux%ux%u!\n",
+            (unsigned)width, (unsigned)height, (unsigned)bits,
+	    (unsigned)samples);
+    TIFFClose(tif);
+    fclose(fp);
+    return (1);
+  }
+
  /*
   * Setup the image size and colorspace...
   */
@@ -1716,5 +1739,5 @@ ImageReadTIFF(image_t    *img,		/* IO - Image */
 
 
 /*
- * End of "$Id: image-tiff.c,v 1.23 2002/04/19 16:17:27 mike Exp $".
+ * End of "$Id: image-tiff.c,v 1.25 2002/12/17 18:59:27 swdev Exp $".
  */
