@@ -1,5 +1,5 @@
 /*
- * "$Id: pstops.c,v 1.93 2002/12/17 18:59:28 swdev Exp $"
+ * "$Id: pstops.c,v 1.102 2003/04/23 17:41:07 mike Exp $"
  *
  *   PostScript filter for the Common UNIX Printing System (CUPS).
  *
@@ -31,7 +31,9 @@
  *   do_prolog()   - Send the necessary document prolog commands...
  *   do_setup()    - Send the necessary document setup commands...
  *   end_nup()     - End processing for N-up printing...
+ *   psbcp()       - Enable the binary communications protocol on the printer.
  *   psgets()      - Get a line from a file.
+ *   pswrite()     - Write data from a file.
  *   start_nup()   - Start processing for N-up printing...
  */
 
@@ -48,26 +50,30 @@
 
 #define MAX_PAGES	10000
 
-#define BORDER_NONE	0	/* No border or hairline border */
-#define BORDER_THICK	1	/* Think border */
-#define BORDER_SINGLE	2	/* Single-line hairline border */
-#define BORDER_SINGLE2	3	/* Single-line thick border */
-#define BORDER_DOUBLE	4	/* Double-line hairline border */
-#define BORDER_DOUBLE2	5	/* Double-line thick border */
+#define BORDER_NONE	0		/* No border or hairline border */
+#define BORDER_THICK	1		/* Think border */
+#define BORDER_SINGLE	2		/* Single-line hairline border */
+#define BORDER_SINGLE2	3		/* Single-line thick border */
+#define BORDER_DOUBLE	4		/* Double-line hairline border */
+#define BORDER_DOUBLE2	5		/* Double-line thick border */
 
-#define LAYOUT_LRBT	0	/* Left to right, bottom to top */
-#define LAYOUT_LRTB	1	/* Left to right, top to bottom */
-#define LAYOUT_RLBT	2	/* Right to left, bottom to top */
-#define LAYOUT_RLTB	3	/* Right to left, top to bottom */
-#define LAYOUT_BTLR	4	/* Bottom to top, left to right */
-#define LAYOUT_TBLR	5	/* Top to bottom, left to right */
-#define LAYOUT_BTRL	6	/* Bottom to top, right to left */
-#define LAYOUT_TBRL	7	/* Top to bottom, right to left */
+#define LAYOUT_LRBT	0		/* Left to right, bottom to top */
+#define LAYOUT_LRTB	1		/* Left to right, top to bottom */
+#define LAYOUT_RLBT	2		/* Right to left, bottom to top */
+#define LAYOUT_RLTB	3		/* Right to left, top to bottom */
+#define LAYOUT_BTLR	4		/* Bottom to top, left to right */
+#define LAYOUT_TBLR	5		/* Top to bottom, left to right */
+#define LAYOUT_BTRL	6		/* Bottom to top, right to left */
+#define LAYOUT_TBRL	7		/* Top to bottom, right to left */
 
-#define LAYOUT_NEGATEY	1
-#define LAYOUT_NEGATEX	2
+#define LAYOUT_NEGATEY	1		/* The bits for the layout */
+#define LAYOUT_NEGATEX	2		/* definitions above... */
 #define LAYOUT_VERTICAL	4
 
+#define PROT_STANDARD	0		/* Adobe standard protocol */
+#define PROT_BCP	1		/* Adobe BCP protocol */
+#define PROT_TBCP	2		/* Adobe TBCP protocol */
+
 
 /*
  * Globals...
@@ -87,7 +93,9 @@ int		Order = 0,		/* 0 = normal, 1 = reverse pages */
 		UseESPsp = 0,		/* Use ESPshowpage? */
 		Border = BORDER_NONE,	/* Border around pages */
 		Layout = LAYOUT_LRTB,	/* Layout of N-up pages */
-		NormalLandscape = 0;	/* Normal rotation for landscape? */
+		NormalLandscape = 0,	/* Normal rotation for landscape? */
+		Protocol = PROT_STANDARD;
+					/* Transmission protocol to use */
 
 
 /*
@@ -103,7 +111,9 @@ static void	end_nup(int number);
 #define		is_first_page(p)	(NUp == 1 || (((p)+1) % NUp) == 1)
 #define		is_last_page(p)		(NUp > 1 && (((p)+1) % NUp) == 0)
 #define 	is_not_last_page(p)	(NUp > 1 && ((p) % NUp) != 0)
-static char	*psgets(char *buf, size_t len, FILE *fp);
+static void	psbcp(ppd_file_t *ppd);
+static char	*psgets(char *buf, size_t *bytes, FILE *fp);
+static size_t	pswrite(const char *buf, size_t bytes, FILE *fp);
 static void	start_nup(int number, int show_border);
 
 
@@ -117,6 +127,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 {
   FILE		*fp;		/* Print file */
   ppd_file_t	*ppd;		/* PPD file */
+  ppd_attr_t	*attr;		/* Attribute in PPD file */
   int		num_options;	/* Number of print options */
   cups_option_t	*options;	/* Print options */
   const char	*val;		/* Option value */
@@ -128,6 +139,7 @@ main(int  argc,			/* I - Number of command-line arguments */
   int		sloworder;	/* 1 if we need to order manually */
   int		slowduplex;	/* 1 if we need an even number of pages */
   char		line[8192];	/* Line buffer */
+  size_t	len;		/* Length of line buffer */
   float		g;		/* Gamma correction value */
   float		b;		/* Brightness factor */
   int		level;		/* Nesting level for embedded files */
@@ -328,6 +340,23 @@ main(int  argc,			/* I - Number of command-line arguments */
   else
     temp = NULL;
 
+ /*
+  * See if we should use a binary transmission protocol...
+  */
+
+  if ((attr = ppdFindAttr(ppd, "cupsProtocol", NULL)) != NULL &&
+      attr->value != NULL)
+  {
+    if (!strcasecmp(attr->value, "TBCP"))
+      Protocol = PROT_TBCP;
+    else if (!strcasecmp(attr->value, "BCP"))
+    {
+      Protocol = PROT_BCP;
+
+      psbcp(ppd);
+    }
+  }
+
  /*
   * Write any "exit server" options that have been selected...
   */
@@ -344,19 +373,54 @@ main(int  argc,			/* I - Number of command-line arguments */
   * Read the first line to see if we have DSC comments...
   */
 
-  if (psgets(line, sizeof(line), fp) == NULL)
+  len = sizeof(line);
+  if (psgets(line, &len, fp) == NULL)
   {
     fputs("ERROR: Empty print file!\n", stderr);
     ppdClose(ppd);
     return (1);
   }
 
+  fwrite(line, 1, len, stdout);
+
+ /*
+  * Handle leading PJL fun...
+  */
+
+  while (strncmp(line, "\033%-12345X", 9) == 0)
+  {
+   /*
+    * Yup, we have leading PJL fun, so copy it until we hit the line
+    * with "ENTER LANGUAGE"...
+    */
+
+    fputs("DEBUG: Skipping PJL header...\n", stderr);
+
+    while (strstr(line, "ENTER LANGUAGE") == NULL)
+    {
+      len = sizeof(line);
+      if (psgets(line, &len, fp) == NULL)
+        break;
+
+      fwrite(line, 1, len, stdout);
+    }
+
+    len = sizeof(line);
+    if (psgets(line, &len, fp) == NULL)
+      break;
+  }
+
+ /*
+  * Switch to TBCP mode as needed...
+  */
+
+  if (Protocol == PROT_TBCP)
+    fputs("\001M", stdout);
+
  /*
   * Start sending the document with any commands needed...
   */
 
-  fputs(line, stdout);
-
   saweof      = 0;
   sent_espsp  = 0;
   sent_prolog = 0;
@@ -411,6 +475,9 @@ main(int  argc,			/* I - Number of command-line arguments */
     UseESPsp = 1;
   }
 
+  fprintf(stderr, "DEBUG: slowcollate=%d, slowduplex=%d, sloworder=%d\n",
+          slowcollate, slowduplex, sloworder);
+
   if (strncmp(line, "%!PS-Adobe-", 11) == 0 && strstr(line, "EPSF") == NULL)
   {
    /*
@@ -422,8 +489,12 @@ main(int  argc,			/* I - Number of command-line arguments */
 
     level = 0;
 
-    while (psgets(line, sizeof(line), fp) != NULL)
+    while (!feof(fp))
     {
+      len = sizeof(line);
+      if (psgets(line, &len, fp) == NULL)
+        break;
+
       if (strncmp(line, "%%", 2) == 0)
         fprintf(stderr, "DEBUG: %d %s", level, line);
       else if (line[0] != '%' && line[0] && !sent_espsp && UseESPsp)
@@ -520,12 +591,12 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    return (1);
 	  }
 
-	  fwrite(line, 1, nbytes, stdout);
+	  pswrite(line, nbytes, stdout);
 	  tbytes -= nbytes;
 	}
       }
       else if (strncmp(line, "%%Pages:", 8) != 0)
-        fputs(line, stdout);
+        pswrite(line, len, stdout);
     }
 
    /*
@@ -610,17 +681,52 @@ main(int  argc,			/* I - Number of command-line arguments */
       {
 	if (!check_range(real_page))
 	{
-	  while (psgets(line, sizeof(line), fp) != NULL)
+	  while (!feof(fp))
+	  {
+	    len = sizeof(line);
+	    if (psgets(line, &len, fp) == NULL)
+	      break;
+
+	    if (strncmp(line, "%%", 2) == 0)
+              fprintf(stderr, "DEBUG: %d %s", level, line);
+
 	    if (strncmp(line, "%%BeginDocument:", 16) == 0 ||
         	strncmp(line, "%%BeginDocument ", 16) == 0)	/* Adobe Acrobat BUG */
               level ++;
-	    else if (strcmp(line, "%%EndDocument") == 0 && level > 0)
+	    else if (strncmp(line, "%%EndDocument", 13) == 0 && level > 0)
               level --;
 	    else if (strncmp(line, "%%Page:", 7) == 0 && level == 0)
 	    {
 	      real_page ++;
 	      break;
 	    }
+	    else if (strncmp(line, "%%BeginBinary:", 14) == 0 ||
+        	     (strncmp(line, "%%BeginData:", 12) == 0 &&
+	              strstr(line, "ASCII") == NULL && strstr(line, "Hex") == NULL))
+	    {
+	     /*
+              * Skip binary data...
+	      */
+
+              tbytes = atoi(strchr(line, ':') + 1);
+
+	      while (tbytes > 0)
+	      {
+		if (tbytes > sizeof(line))
+		  nbytes = fread(line, 1, sizeof(line), fp);
+		else
+		  nbytes = fread(line, 1, tbytes, fp);
+
+        	if (nbytes < 1)
+		{
+		  perror("ERROR: Early end-of-file while reading binary data");
+		  return (1);
+		}
+
+		tbytes -= nbytes;
+	      }
+	    }
+          }
 
           continue;
         }
@@ -636,7 +742,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 	  if (is_first_page(NumPages))
 	  {
 	    if (ppd == NULL || ppd->num_filters == 0)
-	      fprintf(stderr, "PAGE: %d %d\n", page, Copies);
+	      fprintf(stderr, "PAGE: %d %d\n", page, slowcollate ? 1 : Copies);
 
             printf("%%%%Page: %d %d\n", page, page);
 	    page ++;
@@ -678,7 +784,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 	  }
 
           if (!sloworder)
-	    fwrite(line, 1, nbytes, stdout);
+	    pswrite(line, nbytes, stdout);
 
           if (slowcollate || sloworder)
 	    fwrite(line, 1, nbytes, temp);
@@ -694,13 +800,14 @@ main(int  argc,			/* I - Number of command-line arguments */
       else
       {
         if (!sloworder)
-          fputs(line, stdout);
+          pswrite(line, len, stdout);
 
 	if (slowcollate || sloworder)
-	  fputs(line, temp);
+	  fwrite(line, 1, len, temp);
       }
 
-      if (psgets(line, sizeof(line), fp) == NULL)
+      len = sizeof(line);
+      if (psgets(line, &len, fp) == NULL)
         break;
     }
 
@@ -714,14 +821,14 @@ main(int  argc,			/* I - Number of command-line arguments */
         end_nup(NUp - 1);
       }
 
-      if (slowduplex && !(page & 1))
+      if (Duplex && !(page & 1))
       {
        /*
         * Make sure we have an even number of pages...
 	*/
 
 	if (ppd == NULL || ppd->num_filters == 0)
-	  fprintf(stderr, "PAGE: %d %d\n", page, Copies);
+	  fprintf(stderr, "PAGE: %d %d\n", page, slowcollate ? 1 : Copies);
 
         printf("%%%%Page: %d %d\n", page, page);
 	page ++;
@@ -792,16 +899,16 @@ main(int  argc,			/* I - Number of command-line arguments */
         page_count = (NumPages + NUp - 1) / NUp;
 	copy       = 0;
 
+        fprintf(stderr, "DEBUG: page_count=%d\n", page_count);
+
         do
 	{
-	  if (slowduplex && (page_count & 1))
-	  {
-            basepage = page_count - 1;
-	  }
+	  if (Duplex && (page_count & 1))
+            basepage = page_count;
 	  else
-	    basepage = page_count - 1 - slowduplex;
+	    basepage = page_count - 1;
 
-	  for (; basepage >= 0; basepage -= 1 + slowduplex)
+	  for (; basepage >= 0; basepage --)
 	  {
 	    if (ppd == NULL || ppd->num_filters == 0)
 	      fprintf(stderr, "PAGE: %d %d\n", page,
@@ -812,69 +919,28 @@ main(int  argc,			/* I - Number of command-line arguments */
 
 	    ppdEmit(ppd, stdout, PPD_ORDER_PAGE);
 
-	    for (subpage = 0, number = basepage * NUp;
-	         subpage < NUp && number < NumPages;
-		 subpage ++, number ++)
-	    {
-	      start_nup(number, 1);
-	      fseek(temp, Pages[number], SEEK_SET);
-	      copy_bytes(temp, Pages[number + 1] - Pages[number]);
-	      end_nup(number);
-	    }
-
-            if (is_not_last_page(number))
+            if (basepage >= page_count)
 	    {
 	      start_nup(NUp - 1, 0);
+	      puts("showpage");
               end_nup(NUp - 1);
 	    }
-
-            if (slowduplex)
+	    else
 	    {
-              if (number < NumPages)
+	      for (subpage = 0, number = basepage * NUp;
+	           subpage < NUp && number < NumPages;
+		   subpage ++, number ++)
 	      {
-		if (ppd == NULL || ppd->num_filters == 0)
-		  fprintf(stderr, "PAGE: %d %d\n", page,
-	        	  slowcollate ? 1 : Copies);
+		start_nup(number, 1);
+		fseek(temp, Pages[number], SEEK_SET);
+		copy_bytes(temp, Pages[number + 1] - Pages[number]);
+		end_nup(number);
+	      }
 
-        	printf("%%%%Page: %d %d\n", page, page);
-		page ++;
-
-		ppdEmit(ppd, stdout, PPD_ORDER_PAGE);
-
-		for (subpage = 0, number = (basepage + 1) * NUp;
-	             subpage < NUp && number < NumPages;
-		     subpage ++, number ++)
-		{
-		  start_nup(number, 1);
-		  fseek(temp, Pages[number], SEEK_SET);
-		  copy_bytes(temp, Pages[number + 1] - Pages[number]);
-		  end_nup(number);
-		}
-
-        	if (is_not_last_page(number))
-		{
-		  start_nup(NUp - 1, 0);
-        	  end_nup(NUp - 1);
-		}
-              }
-	      else
+              if (is_not_last_page(number))
 	      {
-	       /*
-        	* Make sure we have an even number of pages...
-		*/
-
-		if (ppd == NULL || ppd->num_filters == 0)
-		  fprintf(stderr, "PAGE: %d %d\n", page, slowcollate ? 1 : Copies);
-
-        	printf("%%%%Page: %d %d\n", page, page);
-		page ++;
-		ppdEmit(ppd, stdout, PPD_ORDER_PAGE);
-
 		start_nup(NUp - 1, 0);
-		puts("showpage");
         	end_nup(NUp - 1);
-
-        	basepage = page_count - 1;
 	      }
 	    }
 	  }
@@ -895,11 +961,15 @@ main(int  argc,			/* I - Number of command-line arguments */
     if (UseESPsp)
       puts("userdict/showpage/ESPshowpage load put\n");
 
-    while (psgets(line, sizeof(line), fp) != NULL)
+    while (!feof(fp))
     {
+      len = sizeof(line);
+      if (psgets(line, &len, fp) == NULL)
+        break;
+
       if (strcmp(line, "\004") != 0 &&
           strncmp(line, "%%Pages:", 8) != 0)
-        fputs(line, stdout);
+        pswrite(line, len, stdout);
 
       if (strncmp(line, "%%EOF", 5) == 0)
       {
@@ -925,6 +995,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 	   "userdict/showpage{}put");
 
     puts("%%BeginProlog");
+    WriteLabelProlog(val, PageBottom, PageTop, PageWidth);
     do_prolog(ppd);
     puts("%%EndProlog");
 
@@ -941,14 +1012,17 @@ main(int  argc,			/* I - Number of command-line arguments */
 
     while ((nbytes = fread(line, 1, sizeof(line), fp)) > 0)
     {
-      fwrite(line, 1, nbytes, stdout);
+      pswrite(line, nbytes, stdout);
 
       if (slowcollate)
 	fwrite(line, 1, nbytes, temp);
     }
 
     if (UseESPsp)
+    {
+      WriteLabels(Orientation);
       puts("ESPshowpage");
+    }
 
     if (slowcollate)
     {
@@ -963,7 +1037,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	Copies --;
 
         if (UseESPsp)
+	{
+	  WriteLabels(Orientation);
           puts("ESPshowpage");
+	}
       }
     }
   }
@@ -1097,7 +1174,7 @@ copy_bytes(FILE   *fp,		/* I - File to read from */
 
     nleft -= nbytes;
 
-    fwrite(buffer, 1, nbytes, stdout);
+    pswrite(buffer, nbytes, stdout);
   }
 }
 
@@ -1241,6 +1318,61 @@ end_nup(int number)	/* I - Page number */
 }
 
 
+/*
+ * 'psbcp()' - Enable the binary communications protocol on the printer.
+ */
+
+static void
+psbcp(ppd_file_t *ppd)		/* I - PPD file */
+{
+  if (ppd->jcl_begin)
+    fputs(ppd->jcl_begin, stdout);
+  if (ppd->jcl_ps)
+    fputs(ppd->jcl_ps, stdout);
+
+  if (ppd->language_level == 1)
+  {
+   /*
+    * Use setsoftwareiomode for BCP mode...
+    */
+
+    fputs("%!PS-Adobe-3.0 ExitServer\n", stdout);
+    fputs("%%Title: (BCP - Level 1)\n", stdout);
+    fputs("%%EndComments\n", stdout);
+    fputs("%%BeginExitServer: 0\n", stdout);
+    fputs("serverdict begin 0 exitserver\n", stdout);
+    fputs("%%EndExitServer\n", stdout);
+    fputs("statusdict begin\n", stdout);
+    fputs("/setsoftwareiomode known {100 setsoftwareiomode}\n", stdout);
+    fputs("end\n", stdout);
+    fputs("%EOF\n", stdout);
+  }
+  else
+  {
+   /*
+    * Use setdevparams for BCP mode...
+    */
+
+    fputs("%!PS-Adobe-3.0\n", stdout);
+    fputs("%%Title: (BCP - Level 2)\n", stdout);
+    fputs("%%EndComments\n", stdout);
+    fputs("currentsysparams\n", stdout);
+    fputs("/CurInputDevice 2 copy known {\n", stdout);
+    fputs("get\n", stdout);
+    fputs("<</Protocol /Binary>> setdevparams\n", stdout);
+    fputs("}{\n", stdout);
+    fputs("pop pop\n", stdout);
+    fputs("} ifelse\n", stdout);
+    fputs("%EOF\n", stdout);
+  }
+
+  if (ppd->jcl_end)
+    fputs(ppd->jcl_end, stdout);
+  else if (ppd->num_filters == 0)
+    putchar(0x04);
+}
+
+
 /*
  * 'psgets()' - Get a line from a file.
  *
@@ -1251,16 +1383,17 @@ end_nup(int number)	/* I - Page number */
  *   lines.
  */
 
-static char *		/* O - String or NULL if EOF */
-psgets(char   *buf,	/* I - Buffer to read into */
-       size_t len,	/* I - Length of buffer */
-       FILE   *fp)	/* I - File to read from */
+static char *				/* O  - String or NULL if EOF */
+psgets(char   *buf,			/* I  - Buffer to read into */
+       size_t *bytes,			/* IO - Length of buffer */
+       FILE   *fp)			/* I  - File to read from */
 {
-  char	*bufptr;	/* Pointer into buffer */
-  int	ch;		/* Character from file */
+  char		*bufptr;		/* Pointer into buffer */
+  int		ch;			/* Character from file */
+  size_t	len;			/* Max length of string */
 
 
-  len --;
+  len    = *bytes - 1;
   bufptr = buf;
   ch     = EOF;
 
@@ -1269,20 +1402,24 @@ psgets(char   *buf,	/* I - Buffer to read into */
     if ((ch = getc(fp)) == EOF)
       break;
 
-    if (ch == 0x0d)
+    if (ch == '\r')
     {
      /*
       * Got a CR; see if there is a LF as well...
       */
 
       ch = getc(fp);
-      if (ch != EOF && ch != 0x0a)
-        ungetc(ch, fp);	/* Nope, save it for later... */
 
-      ch = 0x0a;
+      if (ch != EOF && ch != '\n')
+      {
+        ungetc(ch, fp);	/* Nope, save it for later... */
+        ch = '\r';
+      }
+      else
+        *bufptr++ = '\r';
       break;
     }
-    else if (ch == 0x0a)
+    else if (ch == '\n')
       break;
     else
       *bufptr++ = ch;
@@ -1292,14 +1429,20 @@ psgets(char   *buf,	/* I - Buffer to read into */
   * Add a trailing newline if it is there...
   */
 
-  if (ch == '\n')
-    *bufptr++ = '\n';
+  if (ch == '\n' || ch == '\r')
+  {
+    if ((bufptr - buf) < len)
+      *bufptr++ = ch;
+    else
+      ungetc(ch, fp);
+  }
 
  /*
   * Nul-terminate the string and return it (or NULL for EOF).
   */
 
   *bufptr = '\0';
+  *bytes  = bufptr - buf;
 
   if (ch == EOF && bufptr == buf)
     return (NULL);
@@ -1308,19 +1451,86 @@ psgets(char   *buf,	/* I - Buffer to read into */
 }
 
 
+/*
+ * 'pswrite()' - Write data from a file.
+ */
+
+static size_t				/* O - Number of bytes written */
+pswrite(const char *buf,		/* I - Buffer to write */
+        size_t     bytes,		/* I - Bytes to write */
+	FILE       *fp)			/* I - File to write to */
+{
+  size_t	count;			/* Remaining bytes */
+
+
+  switch (Protocol)
+  {
+    case PROT_STANDARD :
+        return (fwrite(buf, 1, bytes, fp));
+
+    case PROT_BCP :
+        for (count = bytes; count > 0; count --, buf ++)
+	  switch (*buf)
+	  {
+	    case 0x01 : /* CTRL-A */
+	    case 0x03 : /* CTRL-C */
+	    case 0x04 : /* CTRL-D */
+	    case 0x05 : /* CTRL-E */
+	    case 0x11 : /* CTRL-Q */
+	    case 0x13 : /* CTRL-S */
+	    case 0x14 : /* CTRL-T */
+	    case 0x1c : /* CTRL-\ */
+	        putchar(0x01);
+		putchar(*buf ^ 0x40);
+	        break;
+
+	    default :
+	        putchar(*buf);
+		break;
+	  }
+        return (bytes);
+
+    case PROT_TBCP :
+        for (count = bytes; count > 0; count --, buf ++)
+	  switch (*buf)
+	  {
+	    case 0x01 : /* CTRL-A */
+	    case 0x03 : /* CTRL-C */
+	    case 0x04 : /* CTRL-D */
+	    case 0x05 : /* CTRL-E */
+	    case 0x11 : /* CTRL-Q */
+	    case 0x13 : /* CTRL-S */
+	    case 0x14 : /* CTRL-T */
+	    case 0x1b : /* CTRL-[ (aka ESC) */
+	    case 0x1c : /* CTRL-\ */
+	        putchar(0x01);
+		putchar(*buf ^ 0x40);
+	        break;
+
+	    default :
+	        putchar(*buf);
+		break;
+	  }
+        return (bytes);
+  }
+
+  return (fwrite(buf, 1, bytes, fp));
+}
+
+
 /*
  * 'start_nup()' - Start processing for N-up printing...
  */
 
 static void
-start_nup(int number,		/* I - Page number */
-          int show_border)	/* I - Show the page border? */
+start_nup(int number,			/* I - Page number */
+          int show_border)		/* I - Show the page border? */
 {
-  int	pos;			/* Position on page */
-  int	x, y;			/* Relative position of subpage */
-  float	w, l,			/* Width and length of subpage */
-	tx, ty;			/* Translation values for subpage */
-  float	pw, pl;			/* Printable width and length of full page */
+  int	pos;				/* Position on page */
+  int	x, y;				/* Relative position of subpage */
+  float	w, l,				/* Width and length of subpage */
+	tx, ty;				/* Translation values for subpage */
+  float	pw, pl;				/* Printable width and length of full page */
 
 
   if (Flip || Orientation || NUp > 1)
@@ -1676,5 +1886,5 @@ start_nup(int number,		/* I - Page number */
 
 
 /*
- * End of "$Id: pstops.c,v 1.93 2002/12/17 18:59:28 swdev Exp $".
+ * End of "$Id: pstops.c,v 1.102 2003/04/23 17:41:07 mike Exp $".
  */
