@@ -1,9 +1,9 @@
 /*
- * "$Id: texttops.c,v 1.45 2002/05/14 01:14:28 mike Exp $"
+ * "$Id: texttops.c,v 1.49 2002/12/17 18:59:30 swdev Exp $"
  *
  *   Text to PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -204,12 +204,8 @@ WriteProlog(const char *title,		/* I - Title of job */
   strftime(curdate, sizeof(curdate), CUPS_STRFTIME_FORMAT, curtm);
 
   puts("%!PS-Adobe-3.0");
-  printf("%%%%BoundingBox: %.0f %.0f %.0f %.0f\n", PageLeft, PageBottom,
-         PageRight, PageTop);
-  if (Orientation & 1)
-    puts("%%Orientation: Landscape");
-  else
-    puts("%%Orientation: Portrait");
+  printf("%%%%BoundingBox: 0 0 %.0f %.0f\n", PageWidth, PageLength);
+  printf("%%cupsRotation: %d\n", (Orientation & 3) * 90);
   puts("%%Creator: texttops/" CUPS_SVERSION);
   printf("%%%%CreationDate: %s\n", curdate);
   printf("%%%%Title: %s\n", title);
@@ -903,17 +899,24 @@ WriteProlog(const char *title,		/* I - Title of job */
 
     puts("/n {");
     puts("\t20 string cvs % convert page number to string");
-    puts("\tdup length % get length");
-    puts("\tdup 2 mul string /P exch def % P = string twice as long");
-    puts("\t0 1 2 index 1 sub { % loop through each character in the page number");
-    puts("\t\tdup 3 index exch get % get character N from the page number");
-    puts("\t\texch 2 mul dup % compute offset in P");
-    puts("\t\tP exch 0 put % font 0");
-    puts("\t\t1 add P exch 2 index put % character");
-    puts("\t\tpop % discard character");
-    puts("\t} for % do for loop");
-    puts("\tpop pop % discard string and length");
-    puts("\tP % put string on stack");
+    if (NumFonts > 1)
+    {
+     /*
+      * Convert a number to double-byte chars...
+      */
+
+      puts("\tdup length % get length");
+      puts("\tdup 2 mul string /P exch def % P = string twice as long");
+      puts("\t0 1 2 index 1 sub { % loop through each character in the page number");
+      puts("\t\tdup 3 index exch get % get character N from the page number");
+      puts("\t\texch 2 mul dup % compute offset in P");
+      puts("\t\tP exch 0 put % font 0");
+      puts("\t\t1 add P exch 2 index put % character");
+      puts("\t\tpop % discard character");
+      puts("\t} for % do for loop");
+      puts("\tpop pop % discard string and length");
+      puts("\tP % put string on stack");
+    }
     puts("} bind def");
 
     printf("/T");
@@ -925,7 +928,7 @@ WriteProlog(const char *title,		/* I - Title of job */
     puts("def");
 
     puts("/H {");
-    puts("gsave");
+    puts("\tgsave");
     puts("\t0.9 setgray");
 
     if (Duplex)
@@ -1304,5 +1307,5 @@ write_text(const char *s)	/* I - String to write */
 
 
 /*
- * End of "$Id: texttops.c,v 1.45 2002/05/14 01:14:28 mike Exp $".
+ * End of "$Id: texttops.c,v 1.49 2002/12/17 18:59:30 swdev Exp $".
  */
