@@ -70,7 +70,7 @@ DisplayFontParam::DisplayFontParam(GString *nameA,
   }
 }
 
-DisplayFontParam::DisplayFontParam(char *nameA, char *xlfdA, char *encodingA) {
+DisplayFontParam::DisplayFontParam(const char *nameA, const char *xlfdA, const char *encodingA) {
   name = new GString(nameA);
   kind = displayFontX;
   x.xlfd = new GString(xlfdA);
@@ -125,7 +125,7 @@ PSFontParam::~PSFontParam() {
 // parsing
 //------------------------------------------------------------------------
 
-GlobalParams::GlobalParams(char *cfgFileName) {
+GlobalParams::GlobalParams(const char *cfgFileName) {
   UnicodeMap *map;
   DisplayFontParam *dfp;
   GString *fileName;
@@ -157,7 +157,7 @@ GlobalParams::GlobalParams(char *cfgFileName) {
   displayCIDFonts = new GHash();
   displayNamedCIDFonts = new GHash();
 #if HAVE_PAPER_H
-  char *paperName;
+  const char *paperName;
   const struct paper *paperType;
   paperinit();
   if ((paperName = systempapername())) {
@@ -174,6 +174,10 @@ GlobalParams::GlobalParams(char *cfgFileName) {
   psPaperWidth = defPaperWidth;
   psPaperHeight = defPaperHeight;
 #endif
+  psLeft = 0;
+  psBottom = 0;
+  psRight = psPaperWidth;
+  psTop = psPaperHeight;
   psDuplex = gFalse;
   psLevel = psLevel2;
   psFile = NULL;
@@ -640,7 +644,7 @@ void GlobalParams::parsePSFont(GList *tokens, GString *fileName, int line) {
   psFonts->add(param->pdfFontName, param);
 }
 
-void GlobalParams::parsePSFont16(char *cmdName, GList *fontList,
+void GlobalParams::parsePSFont16(const char *cmdName, GList *fontList,
 				 GList *tokens, GString *fileName, int line) {
   PSFontParam *param;
   int wMode;
@@ -720,7 +724,7 @@ void GlobalParams::parseInitialZoom(GList *tokens,
   initialZoom = ((GString *)tokens->get(1))->copy();
 }
 
-void GlobalParams::parseFontRastControl(char *cmdName, FontRastControl *val,
+void GlobalParams::parseFontRastControl(const char *cmdName, FontRastControl *val,
 					GList *tokens, GString *fileName,
 					int line) {
   GString *tok;
@@ -737,7 +741,7 @@ void GlobalParams::parseFontRastControl(char *cmdName, FontRastControl *val,
   }
 }
 
-void GlobalParams::parseCommand(char *cmdName, GString **val,
+void GlobalParams::parseCommand(const char *cmdName, GString **val,
 				GList *tokens, GString *fileName, int line) {
   if (tokens->getLength() != 2) {
     error(-1, "Bad '%s' config file command (%s:%d)",
@@ -750,7 +754,7 @@ void GlobalParams::parseCommand(char *cmdName, GString **val,
   *val = ((GString *)tokens->get(1))->copy();
 }
 
-void GlobalParams::parseYesNo(char *cmdName, GBool *flag,
+void GlobalParams::parseYesNo(const char *cmdName, GBool *flag,
 			      GList *tokens, GString *fileName, int line) {
   GString *tok;
 
@@ -822,11 +826,11 @@ GlobalParams::~GlobalParams() {
 // accessors
 //------------------------------------------------------------------------
 
-CharCode GlobalParams::getMacRomanCharCode(char *charName) {
+CharCode GlobalParams::getMacRomanCharCode(const char *charName) {
   return macRomanReverseMap->lookup(charName);
 }
 
-Unicode GlobalParams::mapNameToUnicode(char *charName) {
+Unicode GlobalParams::mapNameToUnicode(const char *charName) {
   return nameToUnicode->lookup(charName);
 }
 
@@ -920,6 +924,16 @@ GString *GlobalParams::getPSFile() {
   return s;
 }
 
+void GlobalParams::getPSImageableArea(int &left, int &bottom, int &right, int &top) {
+  globalParamsLock;
+  left   = psLeft;
+  bottom = psBottom;
+  right  = psRight;
+  top    = psTop;
+  globalParamsUnlock;
+}
+
+
 int GlobalParams::getPSPaperWidth() {
   int w;
 
@@ -1062,7 +1076,7 @@ GBool GlobalParams::getTextKeepTinyChars() {
 }
 
 GString *GlobalParams::findFontFile(GString *fontName,
-				    char *ext1, char *ext2) {
+				    const char *ext1, const char *ext2) {
   GString *dir, *fileName;
   FILE *f;
   int i;
@@ -1207,7 +1221,7 @@ void GlobalParams::addDisplayFont(DisplayFontParam *param) {
   globalParamsUnlock;
 }
 
-void GlobalParams::setPSFile(char *file) {
+void GlobalParams::setPSFile(const char *file) {
   globalParamsLock;
   if (psFile) {
     delete psFile;
@@ -1216,7 +1230,16 @@ void GlobalParams::setPSFile(char *file) {
   globalParamsUnlock;
 }
 
-GBool GlobalParams::setPSPaperSize(char *size) {
+void GlobalParams::setPSImageableArea(int left, int bottom, int right, int top) {
+  globalParamsLock;
+  psLeft   = left;
+  psBottom = bottom;
+  psRight  = right;
+  psTop    = top;
+  globalParamsUnlock;
+}
+
+GBool GlobalParams::setPSPaperSize(const char *size) {
   globalParamsLock;
   if (!strcmp(size, "letter")) {
     psPaperWidth = 612;
@@ -1230,10 +1253,17 @@ GBool GlobalParams::setPSPaperSize(char *size) {
   } else if (!strcmp(size, "A3")) {
     psPaperWidth = 842;
     psPaperHeight = 1190;
+  } else if (!strcmp(size, "Universal")) {
+    psPaperWidth = 595;
+    psPaperHeight = 792;
   } else {
     globalParamsUnlock;
     return gFalse;
   }
+  psLeft = 0;
+  psBottom = 0;
+  psRight = psPaperWidth;
+  psTop = psPaperHeight;
   globalParamsUnlock;
   return gTrue;
 }
@@ -1241,12 +1271,16 @@ GBool GlobalParams::setPSPaperSize(char *size) {
 void GlobalParams::setPSPaperWidth(int width) {
   globalParamsLock;
   psPaperWidth = width;
+  psLeft = 0;
+  psRight = psPaperWidth;
   globalParamsUnlock;
 }
 
 void GlobalParams::setPSPaperHeight(int height) {
   globalParamsLock;
   psPaperHeight = height;
+  psBottom = 0;
+  psTop = psPaperHeight;
   globalParamsUnlock;
 }
 
@@ -1298,14 +1332,14 @@ void GlobalParams::setPSASCIIHex(GBool hex) {
   globalParamsUnlock;
 }
 
-void GlobalParams::setTextEncoding(char *encodingName) {
+void GlobalParams::setTextEncoding(const char *encodingName) {
   globalParamsLock;
   delete textEncoding;
   textEncoding = new GString(encodingName);
   globalParamsUnlock;
 }
 
-GBool GlobalParams::setTextEOL(char *s) {
+GBool GlobalParams::setTextEOL(const char *s) {
   globalParamsLock;
   if (!strcmp(s, "unix")) {
     textEOL = eolUnix;
@@ -1327,14 +1361,14 @@ void GlobalParams::setTextKeepTinyChars(GBool keep) {
   globalParamsUnlock;
 }
 
-void GlobalParams::setInitialZoom(char *s) {
+void GlobalParams::setInitialZoom(const char *s) {
   globalParamsLock;
   delete initialZoom;
   initialZoom = new GString(s);
   globalParamsUnlock;
 }
 
-GBool GlobalParams::setT1libControl(char *s) {
+GBool GlobalParams::setT1libControl(const char *s) {
   GBool ok;
 
   globalParamsLock;
@@ -1343,7 +1377,7 @@ GBool GlobalParams::setT1libControl(char *s) {
   return ok;
 }
 
-GBool GlobalParams::setFreeTypeControl(char *s) {
+GBool GlobalParams::setFreeTypeControl(const char *s) {
   GBool ok;
 
   globalParamsLock;
@@ -1352,7 +1386,7 @@ GBool GlobalParams::setFreeTypeControl(char *s) {
   return ok;
 }
 
-GBool GlobalParams::setFontRastControl(FontRastControl *val, char *s) {
+GBool GlobalParams::setFontRastControl(FontRastControl *val, const char *s) {
   if (!strcmp(s, "none")) {
     *val = fontRastNone;
   } else if (!strcmp(s, "plain")) {
