@@ -44,11 +44,12 @@
 // PostScript prolog and setup
 //------------------------------------------------------------------------
 
-static char *prolog[] = {
+static const char *prolog[] = {
   "/xpdf 75 dict def xpdf begin",
   "% PDF special state",
   "/pdfDictSize 14 def",
   "/pdfSetup {",
+#if 0 /* NOT FOR CUPS */
   "  3 1 roll 2 array astore",
   "  /setpagedevice where {",
   "    pop 3 dict begin",
@@ -60,6 +61,9 @@ static char *prolog[] = {
   "  } {",
   "    pop pop",
   "  } ifelse",
+#else
+  "  pop pop pop",
+#endif /* 0 */
   "} def",
   "/pdfStartPage {",
   "  pdfDictSize dict begin",
@@ -349,7 +353,7 @@ static char *prolog[] = {
   NULL
 };
 
-static char *cmapProlog[] = {
+static const char *cmapProlog[] = {
   "/CIDInit /ProcSet findresource begin",
   "10 dict begin",
   "  begincmap",
@@ -399,11 +403,11 @@ static char *cmapProlog[] = {
 //------------------------------------------------------------------------
 
 struct PSSubstFont {
-  char *psName;			// PostScript name
+  const char *psName;		// PostScript name
   double mWidth;		// width of 'm' character
 };
 
-static char *psFonts[] = {
+static const char *psFonts[] = {
   "Courier",
   "Courier-Bold",
   "Courier-Oblique",
@@ -490,11 +494,11 @@ extern "C" {
 typedef void (*SignalFunc)(int);
 }
 
-static void outputToFile(void *stream, char *data, int len) {
+static void outputToFile(void *stream, const char *data, int len) {
   fwrite(data, 1, len, (FILE *)stream);
 }
 
-PSOutputDev::PSOutputDev(char *fileName, XRef *xrefA, Catalog *catalog,
+PSOutputDev::PSOutputDev(const char *fileName, XRef *xrefA, Catalog *catalog,
 			 int firstPage, int lastPage, PSOutMode modeA) {
   FILE *f;
   PSFileType fileTypeA;
@@ -562,7 +566,7 @@ void PSOutputDev::init(PSOutputFunc outputFuncA, void *outputStreamA,
   PDFRectangle *box;
   Dict *resDict;
   Annots *annots;
-  char **p;
+  const char **p;
   int pg;
   Object obj1, obj2;
   int i;
@@ -621,6 +625,7 @@ void PSOutputDev::init(PSOutputFunc outputFuncA, void *outputStreamA,
     writePSFmt("%%%%DocumentMedia: plain %d %d 0 () ()\n",
 	       paperWidth, paperHeight);
     writePSFmt("%%%%Pages: %d\n", lastPage - firstPage + 1);
+    writePSFmt("%%%%BoundingBox: 0 0 %d %d\n", paperWidth, paperHeight);
     writePS("%%EndComments\n");
     writePS("%%BeginDefaults\n");
     writePS("%%PageMedia: plain\n");
@@ -867,10 +872,10 @@ void PSOutputDev::setupFont(GfxFont *font, Dict *parentResDict) {
   GString *name;
   PSFontParam *fontParam;
   GString *psNameStr;
-  char *psName;
+  const char *psName;
   char type3Name[64], buf[16];
   UnicodeMap *uMap;
-  char *charName;
+  const char *charName;
   double xs, ys;
   int code;
   double w1, w2;
@@ -1081,9 +1086,9 @@ void PSOutputDev::setupFont(GfxFont *font, Dict *parentResDict) {
 	  }
 	}
 	writePS("/");
-	writePSName(charName ? charName : (char *)".notdef");
+	writePSName(charName ? charName : ".notdef");
       }
-      writePS((i == 256-8) ? (char *)"]\n" : (char *)"\n");
+      writePS((i == 256-8) ? "]\n" : "\n");
     }
     writePS("pdfMakeFont\n");
   }
@@ -1093,7 +1098,7 @@ void PSOutputDev::setupFont(GfxFont *font, Dict *parentResDict) {
   }
 }
 
-void PSOutputDev::setupEmbeddedType1Font(Ref *id, char *psName) {
+void PSOutputDev::setupEmbeddedType1Font(Ref *id, const char *psName) {
   static char hexChar[17] = "0123456789abcdef";
   Object refObj, strObj, obj1, obj2;
   Dict *dict;
@@ -1218,7 +1223,7 @@ void PSOutputDev::setupEmbeddedType1Font(Ref *id, char *psName) {
 
 //~ This doesn't handle .pfb files or binary eexec data (which only
 //~ happens in pfb files?).
-void PSOutputDev::setupExternalType1Font(GString *fileName, char *psName) {
+void PSOutputDev::setupExternalType1Font(GString *fileName, const char *psName) {
   FILE *fontFile;
   int c;
   int i;
@@ -1259,7 +1264,7 @@ void PSOutputDev::setupExternalType1Font(GString *fileName, char *psName) {
 }
 
 void PSOutputDev::setupEmbeddedType1CFont(GfxFont *font, Ref *id,
-					  char *psName) {
+					  const char *psName) {
   char *fontBuf;
   int fontLen;
   Type1CFontFile *t1cFile;
@@ -1297,7 +1302,7 @@ void PSOutputDev::setupEmbeddedType1CFont(GfxFont *font, Ref *id,
 }
 
 void PSOutputDev::setupEmbeddedTrueTypeFont(GfxFont *font, Ref *id,
-					    char *psName) {
+					    const char *psName) {
   char *fontBuf;
   int fontLen;
   TrueTypeFontFile *ttFile;
@@ -1339,7 +1344,7 @@ void PSOutputDev::setupEmbeddedTrueTypeFont(GfxFont *font, Ref *id,
   writePS("%%EndResource\n");
 }
 
-void PSOutputDev::setupExternalTrueTypeFont(GfxFont *font, char *psName) {
+void PSOutputDev::setupExternalTrueTypeFont(GfxFont *font, const char *psName) {
   GString *fileName;
   char *fontBuf;
   int fontLen;
@@ -1385,7 +1390,7 @@ void PSOutputDev::setupExternalTrueTypeFont(GfxFont *font, char *psName) {
 }
 
 void PSOutputDev::setupEmbeddedCIDType0Font(GfxFont *font, Ref *id,
-					    char *psName) {
+					    const char *psName) {
   char *fontBuf;
   int fontLen;
   Type1CFontFile *t1cFile;
@@ -1429,7 +1434,7 @@ void PSOutputDev::setupEmbeddedCIDType0Font(GfxFont *font, Ref *id,
 }
 
 void PSOutputDev::setupEmbeddedCIDTrueTypeFont(GfxFont *font, Ref *id,
-					       char *psName) {
+					       const char *psName) {
   char *fontBuf;
   int fontLen;
   TrueTypeFontFile *ttFile;
@@ -1476,7 +1481,7 @@ void PSOutputDev::setupEmbeddedCIDTrueTypeFont(GfxFont *font, Ref *id,
   writePS("%%EndResource\n");
 }
 
-void PSOutputDev::setupType3Font(GfxFont *font, char *psName,
+void PSOutputDev::setupType3Font(GfxFont *font, const char *psName,
 				 Dict *parentResDict) {
   Dict *resDict;
   Dict *charProcs;
@@ -1684,29 +1689,37 @@ void PSOutputDev::setupImage(Ref id, Stream *str) {
 
 void PSOutputDev::startPage(int pageNum, GfxState *state) {
   int x1, y1, x2, y2, width, height, t;
+  int imageWidth, imageHeight;
+  int left, bottom, right, top;
 
+  globalParams->getPSImageableArea(left, bottom, right, top);
+  imageWidth  = right - left;
+  imageHeight = top - bottom;
 
   switch (mode) {
 
   case psModePS:
     writePSFmt("%%%%Page: %d %d\n", pageNum, seqPage);
-    writePS("%%BeginPageSetup\n");
 
     // rotate, translate, and scale page
     x1 = (int)(state->getX1() + 0.5);
     y1 = (int)(state->getY1() + 0.5);
     x2 = (int)(state->getX2() + 0.5);
     y2 = (int)(state->getY2() + 0.5);
+    writePSFmt("%%%%PageBoundingBox: %d %d %d %d\n",
+	       (int)floor(x1), (int)floor(y1),
+	       (int)ceil(x2), (int)ceil(y2));
+    writePS("%%BeginPageSetup\n");
     width = x2 - x1;
     height = y2 - y1;
-    if (width > height && width > paperWidth) {
+    if (width > height && width > imageWidth) {
       landscape = gTrue;
       writePSFmt("%%%%PageOrientation: %s\n",
 		 state->getCTM()[0] ? "Landscape" : "Portrait");
       writePS("pdfStartPage\n");
       writePS("90 rotate\n");
       tx = -x1;
-      ty = -(y1 + paperWidth);
+      ty = -(y1 + imageWidth);
       t = width;
       width = height;
       height = t;
@@ -1718,18 +1731,20 @@ void PSOutputDev::startPage(int pageNum, GfxState *state) {
       tx = -x1;
       ty = -y1;
     }
-    if (width < paperWidth) {
-      tx += (paperWidth - width) / 2;
+    tx += left;
+    ty += bottom;
+    if (width < imageWidth) {
+      tx += (imageWidth - width) / 2;
     }
-    if (height < paperHeight) {
-      ty += (paperHeight - height) / 2;
+    if (height < imageHeight) {
+      ty += (imageHeight - height) / 2;
     }
     if (tx != 0 || ty != 0) {
       writePSFmt("%g %g translate\n", tx, ty);
     }
-    if (width > paperWidth || height > paperHeight) {
-      xScale = (double)paperWidth / (double)width;
-      yScale = (double)paperHeight / (double)height;
+    if (width > imageWidth || height > imageHeight) {
+      xScale = (double)imageWidth / (double)width;
+      yScale = (double)imageHeight / (double)height;
       if (yScale < xScale) {
 	xScale = yScale;
       } else {
@@ -3239,7 +3254,7 @@ void PSOutputDev::writePSChar(char c) {
   }
 }
 
-void PSOutputDev::writePS(char *s) {
+void PSOutputDev::writePS(const char *s) {
   if (t3String) {
     t3String->append(s);
   } else {
@@ -3285,8 +3300,8 @@ void PSOutputDev::writePSString(GString *s) {
   writePSChar(')');
 }
 
-void PSOutputDev::writePSName(char *s) {
-  char *p;
+void PSOutputDev::writePSName(const char *s) {
+  const char *p;
   char c;
 
   p = s;
