@@ -1,5 +1,5 @@
 /*
- * "$Id: gdevcups.c,v 1.59 2002/12/17 19:00:11 swdev Exp $"
+ * "$Id: gdevcups.c,v 1.61 2003/02/05 03:10:21 mike Exp $"
  *
  *   GNU Ghostscript raster output driver for the Common UNIX Printing
  *   System (CUPS).
@@ -1620,7 +1620,7 @@ cups_map_rgb_color(gx_device      *pdev,	/* I - Device info */
 	ciey = 0.212671 * rr + 0.715160 * rg + 0.072169 * rb;
 	ciez = 0.019334 * rr + 0.119193 * rg + 0.950227 * rb;
 
-        if (cups->header.cupsColorSpace != CUPS_CSPACE_CIELab)
+        if (cups->header.cupsColorSpace == CUPS_CSPACE_CIEXYZ)
 	{
 	 /*
 	  * Convert to an integer XYZ color value...
@@ -1629,21 +1629,21 @@ cups_map_rgb_color(gx_device      *pdev,	/* I - Device info */
           if (ciex > 1.0)
 	    ic = 255;
 	  else if (ciex > 0.0)
-	    ic = (int)(ciex * 255.0 + 0.5);
+	    ic = (int)(ciex * 255.0);
 	  else
 	    ic = 0;
 
           if (ciey > 1.0)
 	    im = 255;
 	  else if (ciey > 0.0)
-	    im = (int)(ciey * 255.0 + 0.5);
+	    im = (int)(ciey * 255.0);
 	  else
 	    im = 0;
 
           if (ciez > 1.0)
 	    iy = 255;
 	  else if (ciez > 0.0)
-	    iy = (int)(ciez * 255.0 + 0.5);
+	    iy = (int)(ciez * 255.0);
 	  else
 	    iy = 0;
 	}
@@ -1660,35 +1660,44 @@ cups_map_rgb_color(gx_device      *pdev,	/* I - Device info */
 	  else
 	    ciel = 903.3 * ciey_yn;
 
-          if (ciel < 0.0)
-	    ic = 0;
-	  else if (ciel < 255.0)
-	    ic = (int)(ciel + 0.5);
-	  else
-	    ic = 255;
-
 	  ciea = 500 * (cups_map_cielab(ciex, D65_X) -
 	                cups_map_cielab(ciey, D65_Y));
 	  cieb = 200 * (cups_map_cielab(ciey, D65_Y) -
 	                cups_map_cielab(ciez, D65_Z));
 
-          if (ciea < -127.0)
-	    im = 128;
-	  else if (ciea < 0.0)
-	    im = (int)(ciea + 256.5);
-	  else if (ciea > 127.0)
-	    im = 127;
-	  else
-	    im = (int)(ciea + 0.5);
+         /*
+	  * Scale the L value and bias the a and b values by 128
+	  * so that all values are in the range of 0 to 255.
+	  */
 
-          if (cieb < -127.0)
-	    iy = 128;
-	  else if (cieb < 0.0)
-	    iy = (int)(cieb + 256.5);
-	  else if (cieb > 127.0)
-	    iy = 127;
+	  ciel *= 2.55;
+	  ciea += 128;
+	  cieb += 128;
+
+         /*
+	  * Convert to 8-bit values...
+	  */
+
+          if (ciel < 0.0)
+	    ic = 0;
+	  else if (ciel < 255.0)
+	    ic = ciel;
 	  else
-	    iy = (int)(cieb + 0.5);
+	    ic = 255;
+
+          if (ciea < 0.0)
+	    im = 0;
+	  else if (ciea < 255.0)
+	    im = ciea;
+	  else
+	    im = 255;
+
+          if (cieb < 0.0)
+	    iy = 0;
+	  else if (cieb < 255.0)
+	    iy = cieb;
+	  else
+	    iy = 255;
 	}
 
        /*
@@ -2367,14 +2376,7 @@ cups_set_color_info(gx_device *pdev)	/* I - Device info */
 	 if (cups->header.cupsColorOrder != CUPS_ORDER_CHUNKED)
            cups->header.cupsBitsPerPixel = cups->header.cupsBitsPerColor;
 	 else
-	 {
-	   if (cups->header.cupsColorSpace < CUPS_CSPACE_ICC1)
-	     cups->header.cupsBitsPerPixel = 3 * cups->header.cupsBitsPerColor;
-	   else
-	     cups->header.cupsBitsPerPixel =
-	         (cups->header.cupsColorSpace - CUPS_CSPACE_ICC1 + 1) *
-		 cups->header.cupsBitsPerColor;
-         }
+           cups->header.cupsBitsPerPixel = 3 * cups->header.cupsBitsPerColor;
 
 	 cups->color_info.depth          = 24;
 	 cups->color_info.num_components = 3;
@@ -3640,5 +3642,5 @@ cups_print_planar(gx_device_printer *pdev,	/* I - Printer device */
 
 
 /*
- * End of "$Id: gdevcups.c,v 1.59 2002/12/17 19:00:11 swdev Exp $".
+ * End of "$Id: gdevcups.c,v 1.61 2003/02/05 03:10:21 mike Exp $".
  */
