@@ -1,5 +1,5 @@
 /*
- * "$Id: auth.c,v 1.65 2002/12/17 19:00:12 swdev Exp $"
+ * "$Id: auth.c,v 1.70 2003/04/10 12:57:43 mike Exp $"
  *
  *   Authorization routines for the Common UNIX Printing System (CUPS).
  *
@@ -96,9 +96,9 @@ static void		to64(char *s, unsigned long v, int n);
  * Local globals...
  */
 
-#ifdef __hpux
+#if defined(__hpux) && defined(HAVE_LIBPAM)
 static client_t		*auth_client;	/* Current client being authenticated */
-#endif /* __hpux */
+#endif /* __hpux && HAVE_LIBPAM */
 
 
 /*
@@ -658,7 +658,7 @@ FindBest(const char   *path,	/* I - Resource path */
 		*best;		/* Best match for location so far */
   int		bestlen;	/* Length of best match */
   int		limit;		/* Limit field */
-  static int	limits[] =	/* Map http_status_t to AUTH_LIMIT_xyz */
+  static const int limits[] =	/* Map http_status_t to AUTH_LIMIT_xyz */
 		{
 		  AUTH_LIMIT_ALL,
 		  AUTH_LIMIT_OPTIONS,
@@ -765,21 +765,30 @@ GetMD5Passwd(const char *username,	/* I - Username */
              const char *group,		/* I - Group */
              char       passwd[33])	/* O - MD5 password string */
 {
-  FILE	*fp;				/* passwd.md5 file */
-  char	filename[1024],			/* passwd.md5 filename */
-	line[256],			/* Line from file */
-	tempuser[33],			/* User from file */
-	tempgroup[33];			/* Group from file */
+  cups_file_t	*fp;			/* passwd.md5 file */
+  char		filename[1024],		/* passwd.md5 filename */
+		line[256],		/* Line from file */
+		tempuser[33],		/* User from file */
+		tempgroup[33];		/* Group from file */
 
 
+  LogMessage(L_DEBUG2, "GetMD5Passwd(username=\"%s\", group=\"%s\", passwd=%p)",
+             username, group ? group : "(null)", passwd);
+
   snprintf(filename, sizeof(filename), "%s/passwd.md5", ServerRoot);
-  if ((fp = fopen(filename, "r")) == NULL)
+  if ((fp = cupsFileOpen(filename, "r")) == NULL)
+  {
+    LogMessage(L_ERROR, "Unable to open %s - %s", filename, strerror(errno));
     return (NULL);
+  }
 
-  while (fgets(line, sizeof(line), fp) != NULL)
+  while (cupsFileGets(fp, line, sizeof(line)) != NULL)
   {
     if (sscanf(line, "%32[^:]:%32[^:]:%32s", tempuser, tempgroup, passwd) != 3)
+    {
+      LogMessage(L_ERROR, "Bad MD5 password line: %s", line);
       continue;
+    }
 
     if (strcmp(username, tempuser) == 0 &&
         (group == NULL || strcmp(group, tempgroup) == 0))
@@ -788,7 +797,10 @@ GetMD5Passwd(const char *username,	/* I - Username */
       * Found the password entry!
       */
 
-      fclose(fp);
+      LogMessage(L_DEBUG2, "Found MD5 user %s, group %s...", username,
+                 tempgroup);
+
+      cupsFileClose(fp);
       return (passwd);
     }
   }
@@ -797,7 +809,7 @@ GetMD5Passwd(const char *username,	/* I - Username */
   * Didn't find a password entry - return NULL!
   */
 
-  fclose(fp);
+  cupsFileClose(fp);
   return (NULL);
 }
 
@@ -834,7 +846,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
   struct spwd	*spw;		/* Shadow password data */
 #  endif /* HAVE_SHADOW_H */
 #endif /* HAVE_LIBPAM */
-  static const char *states[] =	/* HTTP client states... */
+  static const char * const states[] =	/* HTTP client states... */
 		{
 		  "WAITING",
 		  "OPTIONS",
@@ -930,7 +942,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
   if (auth == AUTH_DENY && best->satisfy == AUTH_SATISFY_ALL)
     return (HTTP_FORBIDDEN);
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
  /*
   * See if encryption is required...
   */
@@ -940,7 +952,7 @@ IsAuthorized(client_t *con)	/* I - Connection */
     LogMessage(L_DEBUG2, "IsAuthorized: Need upgrade to TLS...");
     return (HTTP_UPGRADE_REQUIRED);
   }
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 
  /*
   * Now see what access level is required...
@@ -1153,6 +1165,8 @@ IsAuthorized(client_t *con)	/* I - Connection */
 
 	  if (best->num_names && best->level == AUTH_GROUP)
 	  {
+	    LogMessage(L_DEBUG2, "IsAuthorized: num_names = %d", best->num_names);
+
             for (i = 0; i < best->num_names; i ++)
 	      if (GetMD5Passwd(con->username, best->names[i], md5))
 		break;
@@ -1188,6 +1202,8 @@ IsAuthorized(client_t *con)	/* I - Connection */
 
 	  if (best->num_names && best->level == AUTH_GROUP)
 	  {
+	    LogMessage(L_DEBUG2, "IsAuthorized: num_names = %d", best->num_names);
+
             for (i = 0; i < best->num_names; i ++)
 	      if (GetMD5Passwd(con->username, best->names[i], md5))
 		break;
@@ -1627,5 +1643,5 @@ to64(char          *s,	/* O - Output string */
 
 
 /*
- * End of "$Id: auth.c,v 1.65 2002/12/17 19:00:12 swdev Exp $".
+ * End of "$Id: auth.c,v 1.70 2003/04/10 12:57:43 mike Exp $".
  */
