@@ -1,5 +1,5 @@
 /*
- * "$Id: client.h,v 1.20 2002/12/17 19:00:14 swdev Exp $"
+ * "$Id: client.h,v 1.26 2003/03/31 16:31:36 mike Exp $"
  *
  *   Client definitions for the Common UNIX Printing System (CUPS) scheduler.
  *
@@ -37,8 +37,8 @@ typedef struct
   char		username[33],		/* Username from Authorization: line */
 		password[33],		/* Password from Authorization: line */
 		uri[HTTP_MAX_URI],	/* Localized URL/URI for GET/PUT */
-		filename[HTTP_MAX_URI],	/* Filename of output file */
-		command[HTTP_MAX_URI],	/* Command to run */
+		*filename,		/* Filename of output file */
+		*command,		/* Command to run */
 		*options;		/* Options for command */
   int		file;			/* Input/output file */
   int		pipe_pid;		/* Pipe process ID (or 0 if not a pipe) */
@@ -69,13 +69,15 @@ typedef struct
 VAR int			ListenBackLog	VALUE(SOMAXCONN);
 VAR int			NumListeners	VALUE(0);
 					/* Number of listening sockets */
-VAR listener_t		Listeners[MAX_LISTENERS];
+VAR listener_t		*Listeners	VALUE(NULL);
 					/* Listening sockets */
 VAR int			NumClients	VALUE(0);
 					/* Number of HTTP clients */
 VAR client_t		*Clients	VALUE(NULL);
 					/* HTTP clients */
 VAR struct sockaddr_in	ServerAddr;	/* Server IP address */
+VAR int			CGIPipes[2]	VALUE2(-1,-1);
+					/* Pipes for CGI error/debug output */
 
 
 /*
@@ -86,8 +88,10 @@ extern void	AcceptClient(listener_t *lis);
 extern void	CloseAllClients(void);
 extern void	CloseClient(client_t *con);
 extern int	EncryptClient(client_t *con);
+extern int	IsCGI(client_t *con, const char *filename,
+		      struct stat *filestats, mime_type_t *type);
 extern void	PauseListening(void);
-extern void	ProcessIPPRequest(client_t *con);
+extern int	ProcessIPPRequest(client_t *con);
 extern int	ReadClient(client_t *con);
 extern void	ResumeListening(void);
 extern int	SendCommand(client_t *con, char *command, char *options);
@@ -98,8 +102,9 @@ extern int	SendHeader(client_t *con, http_status_t code, char *type);
 extern void	ShutdownClient(client_t *con);
 extern void	StartListening(void);
 extern void	StopListening(void);
+extern void	UpdateCGI(void);
 extern int	WriteClient(client_t *con);
 
 /*
- * End of "$Id: client.h,v 1.20 2002/12/17 19:00:14 swdev Exp $".
+ * End of "$Id: client.h,v 1.26 2003/03/31 16:31:36 mike Exp $".
  */
