@@ -1,5 +1,5 @@
 /*
- * "$Id: cups-lpd.c,v 1.36 2002/12/17 19:00:15 swdev Exp $"
+ * "$Id: cups-lpd.c,v 1.39 2003/05/22 12:48:14 mike Exp $"
  *
  *   Line Printer Daemon interface for the Common UNIX Printing System (CUPS).
  *
@@ -446,19 +446,30 @@ recv_print_job(const char    *dest,	/* I - Destination */
   num_dests = cupsGetDests(&dests);
   if ((destptr = cupsGetDest(queue, instance, num_dests, dests)) == NULL)
   {
-    if (instance)
-      syslog(LOG_ERR, "Unknown destination %s/%s!", queue, instance);
-    else
-      syslog(LOG_ERR, "Unknown destination %s!", queue);
+   /*
+    * If the queue name is blank or "lp" then use the default queue.
+    */
 
-    cupsFreeDests(num_dests, dests);
+    if (!queue[0] || !strcmp(queue, "lp"))
+      if ((destptr = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+	strlcpy(queue, destptr->name, sizeof(queue));
 
-    putchar(1);
+    if (destptr == NULL)
+    {
+      if (instance)
+	syslog(LOG_ERR, "Unknown destination %s/%s!", queue, instance);
+      else
+	syslog(LOG_ERR, "Unknown destination %s!", queue);
 
-    return (1);
+      cupsFreeDests(num_dests, dests);
+
+      putchar(1);
+
+      return (1);
+    }
   }
-  else
-    putchar(0);
+
+  putchar(0);
 
   while (smart_gets(line, sizeof(line), stdin) != NULL)
   {
@@ -925,7 +936,7 @@ send_state(const char *dest,		/* I - Destination */
   char		uri[HTTP_MAX_URI];	/* Printer URI */
   char		queue[256],		/* Printer/class queue */
 		*instance;		/* Printer/class instance */
-  static const char *ranks[10] =	/* Ranking strings */
+  static const char * const ranks[10] =	/* Ranking strings */
 		{
 		  "th",
 		  "st",
@@ -938,8 +949,8 @@ send_state(const char *dest,		/* I - Destination */
 		  "th",
 		  "th"
 		};
-  static const char *requested[] =	/* Requested attributes */
-		{
+  static const char * const requested[] =
+		{			/* Requested attributes */
 		  "job-id",
 		  "job-k-octets",
 		  "job-state",
@@ -1205,11 +1216,11 @@ send_state(const char *dest,		/* I - Destination */
 	else
 	  strlcpy(namestr, jobname, sizeof(namestr));
 
-        printf("%s: %-34.34s[job %d localhost]\n", jobuser, rankstr, jobid);
-        printf("        %-40.40s%d bytes\n", namestr, jobsize);
+        printf("%s: %-33.33s [job %d localhost]\n", jobuser, rankstr, jobid);
+        printf("        %-39.39s %d bytes\n", namestr, jobsize);
       }
       else
-        printf("%-7s %-8.8s%-8d%-32.32s%d bytes\n", rankstr, jobuser,
+        printf("%-7s %-7.7s %-7d %-31.31s %d bytes\n", rankstr, jobuser,
 	       jobid, jobname, jobsize);
 
       if (attr == NULL)
@@ -1289,5 +1300,5 @@ smart_gets(char *s,	/* I - Pointer to line buffer */
 
 
 /*
- * End of "$Id: cups-lpd.c,v 1.36 2002/12/17 19:00:15 swdev Exp $".
+ * End of "$Id: cups-lpd.c,v 1.39 2003/05/22 12:48:14 mike Exp $".
  */
