@@ -1,5 +1,5 @@
 /*
- * "$Id: listen.c,v 1.14 2002/12/17 19:00:17 swdev Exp $"
+ * "$Id: listen.c,v 1.18 2003/03/31 16:31:36 mike Exp $"
  *
  *   Server listening routines for the Common UNIX Printing System (CUPS)
  *   scheduler.
@@ -48,7 +48,7 @@ PauseListening(void)
   listener_t	*lis;		/* Current listening socket */
 
 
-  if (!FD_ISSET(Listeners[0].fd, &InputSet))
+  if (NumListeners < 1 || !FD_ISSET(Listeners[0].fd, InputSet))
     return;
 
   if (NumClients == MaxClients)
@@ -61,7 +61,7 @@ PauseListening(void)
     LogMessage(L_DEBUG2, "PauseListening: Removing fd %d from InputSet...",
                lis->fd);
 
-    FD_CLR(lis->fd, &InputSet);
+    FD_CLR(lis->fd, InputSet);
   }
 }
 
@@ -77,7 +77,7 @@ ResumeListening(void)
   listener_t	*lis;		/* Current listening socket */
 
 
-  if (FD_ISSET(Listeners[0].fd, &InputSet))
+  if (NumListeners < 1 || FD_ISSET(Listeners[0].fd, InputSet))
     return;
 
   if (NumClients >= (MaxClients - 1))
@@ -90,7 +90,7 @@ ResumeListening(void)
     LogMessage(L_DEBUG2, "ResumeListening: Adding fd %d to InputSet...",
                lis->fd);
 
-    FD_SET(lis->fd, &InputSet);
+    FD_SET(lis->fd, InputSet);
   }
 }
 
@@ -131,8 +131,8 @@ StartListening(void)
     * Didn't find it!  Use an address of 0...
     */
 
-    LogMessage(L_ERROR, "StartListening: Unable to find IP address for server name \"%s\"!\n",
-               ServerName);
+    LogMessage(L_ERROR, "StartListening: Unable to find IP address for server name \"%s\" - %s\n",
+               ServerName, hstrerror(h_errno));
 
     ServerAddr.sin_family = AF_INET;
   }
@@ -144,7 +144,7 @@ StartListening(void)
   for (i = NumListeners, lis = Listeners; i > 0; i --, lis ++)
   {
     LogMessage(L_DEBUG, "StartListening: address=%08x port=%d",
-               ntohl(lis->address.sin_addr.s_addr),
+               (unsigned)ntohl(lis->address.sin_addr.s_addr),
 	       ntohs(lis->address.sin_port));
 
     if ((lis->fd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
@@ -218,5 +218,5 @@ StopListening(void)
 
 
 /*
- * End of "$Id: listen.c,v 1.14 2002/12/17 19:00:17 swdev Exp $".
+ * End of "$Id: listen.c,v 1.18 2003/03/31 16:31:36 mike Exp $".
  */
