@@ -1,5 +1,5 @@
 /*
- * "$Id: log.c,v 1.27 2002/12/17 19:00:17 swdev Exp $"
+ * "$Id: log.c,v 1.32 2003/04/10 20:14:06 mike Exp $"
  *
  *   Log file routines for the Common UNIX Printing System (CUPS).
  *
@@ -46,7 +46,7 @@
  * Local functions...
  */
 
-static int	check_log_file(FILE **, const char *);
+static int	check_log_file(cups_file_t **, const char *);
 
 
 /*
@@ -58,8 +58,8 @@ GetDateTime(time_t t)		/* I - Time value */
 {
   struct tm	*date;		/* Date/time value */
   static char	s[1024];	/* Date/time string */
-  static const char *months[12] =/* Months */
-		{
+  static const char * const months[12] =
+		{		/* Months */
 		  "Jan",
 		  "Feb",
 		  "Mar",
@@ -116,7 +116,7 @@ LogMessage(int        level,	/* I - Log level */
   int		len;		/* Length of message */
   char		line[1024];	/* Line for output file */
   va_list	ap;		/* Argument pointer */
-  static char	levels[] =	/* Log levels... */
+  static const char levels[] =	/* Log levels... */
 		{
 		  ' ',
 		  'X',
@@ -130,7 +130,7 @@ LogMessage(int        level,	/* I - Log level */
 		  'd'
 		};
 #ifdef HAVE_VSYSLOG
-  static int	syslevels[] =	/* SYSLOG levels... */
+  static const int syslevels[] =/* SYSLOG levels... */
 		{
 		  0,
 		  LOG_EMERG,
@@ -153,6 +153,8 @@ LogMessage(int        level,	/* I - Log level */
   if (level > LogLevel)
     return (1);
 
+  HoldSignals();
+
 #ifdef HAVE_VSYSLOG
  /*
   * See if we are logging errors via syslog...
@@ -164,6 +166,8 @@ LogMessage(int        level,	/* I - Log level */
     vsyslog(syslevels[level], message, ap);
     va_end(ap);
 
+    ReleaseSignals();
+
     return (1);
   }
 #endif /* HAVE_VSYSLOG */
@@ -173,13 +177,17 @@ LogMessage(int        level,	/* I - Log level */
   */
 
   if (!check_log_file(&ErrorFile, ErrorLog))
+  {
+    ReleaseSignals();
+
     return (0);
-  
+  }
+
  /*
   * Print the log level and date/time...
   */
 
-  fprintf(ErrorFile, "%c %s ", levels[level], GetDateTime(time(NULL)));
+  cupsFilePrintf(ErrorFile, "%c %s ", levels[level], GetDateTime(time(NULL)));
 
  /*
   * Then the log message...
@@ -193,11 +201,14 @@ LogMessage(int        level,	/* I - Log level */
   * Then a newline...
   */
 
-  fputs(line, ErrorFile);
   if (len > 0 && line[len - 1] != '\n')
-    putc('\n', ErrorFile);
+    cupsFilePrintf(ErrorFile, "%s\n", line);
+  else
+    cupsFilePuts(ErrorFile, line);
 
-  fflush(ErrorFile);
+  cupsFileFlush(ErrorFile);
+
+  ReleaseSignals();
 
   return (1);
 }
@@ -211,11 +222,15 @@ int				/* O - 1 on success, 0 on error */
 LogPage(job_t       *job,	/* I - Job being printed */
         const char  *page)	/* I - Page being printed */
 {
-  ipp_attribute_t *billing;	/* job-billing attribute */
+  ipp_attribute_t *billing,	/* job-billing attribute */
+                  *hostname;	/* job-originating-host-name attribute */
 
 
+  billing  = ippFindAttribute(job->attrs, "job-billing", IPP_TAG_ZERO);
+  hostname = ippFindAttribute(job->attrs, "job-originating-host-name",
+                              IPP_TAG_ZERO);
 
-  billing = ippFindAttribute(job->attrs, "job-billing", IPP_TAG_ZERO);
+  HoldSignals();
 
 #ifdef HAVE_VSYSLOG
  /*
@@ -224,8 +239,12 @@ LogPage(job_t       *job,	/* I - Job being printed */
 
   if (strcmp(PageLog, "syslog") == 0)
   {
-    syslog(LOG_INFO, "PAGE %s %s %d %s %s", job->printer->name, job->username,
-           job->id, page, billing ? billing->values[0].string.text : "");
+    syslog(LOG_INFO, "PAGE %s %s %d %s %s %s", job->printer->name,
+           job->username ? job->username : "-",
+           job->id, page, billing ? billing->values[0].string.text : "-",
+           hostname->values[0].string.text);
+
+    ReleaseSignals();
 
     return (1);
   }
@@ -236,18 +255,27 @@ LogPage(job_t       *job,	/* I - Job being printed */
   */
 
   if (!check_log_file(&PageFile, PageLog))
+  {
+    ReleaseSignals();
+
     return (0);
+  }
 
  /*
   * Print a page log entry of the form:
   *
-  *    printer job-id user [DD/MON/YYYY:HH:MM:SS +TTTT] page num-copies billing
+  *    printer job-id user [DD/MON/YYYY:HH:MM:SS +TTTT] page num-copies \
+  *        billing hostname
   */
 
-  fprintf(PageFile, "%s %s %d %s %s %s\n", job->printer->name, job->username,
-          job->id, GetDateTime(time(NULL)), page,
-	  billing ? billing->values[0].string.text : "");
-  fflush(PageFile);
+  cupsFilePrintf(PageFile, "%s %s %d %s %s %s %s\n", job->printer->name,
+        	 job->username ? job->username : "-",
+        	 job->id, GetDateTime(time(NULL)), page,
+		 billing ? billing->values[0].string.text : "-",
+        	 hostname->values[0].string.text);
+  cupsFileFlush(PageFile);
+
+  ReleaseSignals();
 
   return (1);
 }
@@ -261,8 +289,8 @@ int				/* O - 1 on success, 0 on error */
 LogRequest(client_t      *con,	/* I - Request to log */
            http_status_t code)	/* I - Response code */
 {
-  static const char *states[] =	/* HTTP client states... */
-		{
+  static const char * const states[] =
+		{		/* HTTP client states... */
 		  "WAITING",
 		  "OPTIONS",
 		  "GET",
@@ -280,6 +308,8 @@ LogRequest(client_t      *con,	/* I - Request to log */
 		};
 
 
+  HoldSignals();
+
 #ifdef HAVE_VSYSLOG
  /*
   * See if we are logging accesses via syslog...
@@ -293,6 +323,8 @@ LogRequest(client_t      *con,	/* I - Request to log */
 	   con->http.version / 100, con->http.version % 100,
 	   code, con->bytes);
 
+    ReleaseSignals();
+
     return (1);
   }
 #endif /* HAVE_VSYSLOG */
@@ -302,18 +334,24 @@ LogRequest(client_t      *con,	/* I - Request to log */
   */
 
   if (!check_log_file(&AccessFile, AccessLog))
+  {
+    ReleaseSignals();
+
     return (0);
+  }
 
  /*
   * Write a log of the request in "common log format"...
   */
 
-  fprintf(AccessFile, "%s - %s %s \"%s %s HTTP/%d.%d\" %d %d\n",
-          con->http.hostname, con->username[0] != '\0' ? con->username : "-",
-	  GetDateTime(con->start), states[con->operation], con->uri,
-	  con->http.version / 100, con->http.version % 100,
-	  code, con->bytes);
-  fflush(AccessFile);
+  cupsFilePrintf(AccessFile, "%s - %s %s \"%s %s HTTP/%d.%d\" %d %d\n",
+        	 con->http.hostname, con->username[0] != '\0' ? con->username : "-",
+		 GetDateTime(con->start), states[con->operation], con->uri,
+		 con->http.version / 100, con->http.version % 100,
+		 code, con->bytes);
+  cupsFileFlush(AccessFile);
+
+  ReleaseSignals();
 
   return (1);
 }
@@ -324,8 +362,8 @@ LogRequest(client_t      *con,	/* I - Request to log */
  */
 
 static int				/* O  - 1 if log file open */
-check_log_file(FILE       **log,	/* IO - Log file */
-	       const char *logname)	/* I  - Log filename */
+check_log_file(cups_file_t **log,	/* IO - Log file */
+	       const char  *logname)	/* I  - Log filename */
 {
   char	backname[1024],			/* Backup log filename */
 	filename[1024],			/* Formatted log filename */
@@ -344,7 +382,7 @@ check_log_file(FILE       **log,	/* IO - Log file */
   */
 
   if (*log == NULL ||
-      (ftell(*log) > MaxLogSize && MaxLogSize > 0))
+      (cupsFileTell(*log) > MaxLogSize && MaxLogSize > 0))
   {
    /*
     * Handle format strings...
@@ -404,24 +442,27 @@ check_log_file(FILE       **log,	/* IO - Log file */
     * Nope, open the log file...
     */
 
-    if ((*log = fopen(filename, "a")) == NULL)
+    if ((*log = cupsFileOpen(filename, "a")) == NULL)
       return (0);
 
-    fchown(fileno(*log), User, Group);
-    fchmod(fileno(*log), LogFilePerm);
+    if (strncmp(filename, "/dev/", 5))
+    {
+      fchown(cupsFileNumber(*log), User, Group);
+      fchmod(cupsFileNumber(*log), LogFilePerm);
+    }
   }
 
  /*
   * Do we need to rotate the log?
   */
 
-  if (ftell(*log) > MaxLogSize && MaxLogSize > 0)
+  if (cupsFileTell(*log) > MaxLogSize && MaxLogSize > 0)
   {
    /*
     * Rotate log file...
     */
 
-    fclose(*log);
+    cupsFileClose(*log);
 
     strcpy(backname, filename);
     strlcat(backname, ".O", sizeof(backname));
@@ -429,11 +470,14 @@ check_log_file(FILE       **log,	/* IO - Log file */
     unlink(backname);
     rename(filename, backname);
 
-    if ((*log = fopen(filename, "a")) == NULL)
+    if ((*log = cupsFileOpen(filename, "a")) == NULL)
       return (0);
 
-    fchown(fileno(*log), User, Group);
-    fchmod(fileno(*log), LogFilePerm);
+    if (strncmp(filename, "/dev/", 5))
+    {
+      fchown(cupsFileNumber(*log), User, Group);
+      fchmod(cupsFileNumber(*log), LogFilePerm);
+    }
   }
 
   return (1);
@@ -441,5 +485,5 @@ check_log_file(FILE       **log,	/* IO - Log file */
 
 
 /*
- * End of "$Id: log.c,v 1.27 2002/12/17 19:00:17 swdev Exp $".
+ * End of "$Id: log.c,v 1.32 2003/04/10 20:14:06 mike Exp $".
  */
