@@ -1,5 +1,5 @@
 /*
- * "$Id: main.c,v 1.85 2002/12/17 19:00:17 swdev Exp $"
+ * "$Id: main.c,v 1.104 2003/05/09 15:11:13 mike Exp $"
  *
  *   Scheduler main loop for the Common UNIX Printing System (CUPS).
  *
@@ -25,7 +25,12 @@
  *
  *   main()               - Main entry for the CUPS scheduler.
  *   CatchChildSignals()  - Catch SIGCHLD signals...
+ *   HoldSignals()        - Hold child and termination signals.
  *   IgnoreChildSignals() - Ignore SIGCHLD signals...
+ *   ReleaseSignals()     - Release signals for delivery.
+ *   SetString()          - Set a string value.
+ *   SetStringf()         - Set a formatted string value.
+ *   parent_handler()     - Catch USR1/CHLD signals...
  *   sigchld_handler()    - Handle 'child' signals from old processes.
  *   sighup_handler()     - Handle 'hangup' signals to reconfigure the scheduler.
  *   sigterm_handler()    - Handle 'terminate' signals that stop the scheduler.
@@ -46,35 +51,42 @@
 #  include <malloc.h>
 #endif /* HAVE_MALLOC_H && HAVE_MALLINFO */
 
-#ifndef FD_SETSIZE
-#  define FD_SETSIZE	1024
-#endif /* !FD_SETSIZE */
-
 
 /*
  * Local functions...
  */
 
+static void	parent_handler(int sig);
 static void	sigchld_handler(int sig);
 static void	sighup_handler(int sig);
 static void	sigterm_handler(int sig);
-static void	sigusr1_handler(int sig);
 static void	usage(void);
 
 
+/*
+ * Local globals...
+ */
+
+static int	parent_signal = 0;	/* Set to signal number from child */
+static int	holdcount = 0;		/* Number of time "hold" was called */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+static sigset_t	holdmask;		/* Old POSIX signal mask */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
+
+
 /*
  * 'main()' - Main entry for the CUPS scheduler.
  */
 
-int				/* O - Exit status */
-main(int  argc,			/* I - Number of command-line arguments */
-     char *argv[])		/* I - Command-line arguments */
+int					/* O - Exit status */
+main(int  argc,				/* I - Number of command-line arguments */
+     char *argv[])			/* I - Command-line arguments */
 {
   int			i;		/* Looping var */
   char			*opt;		/* Option character */
   int			fg;		/* Run in the foreground */
-  fd_set		input,		/* Input set for select() */
-			output;		/* Output set for select() */
+  fd_set		*input,		/* Input set for select() */
+			*output;	/* Output set for select() */
   client_t		*con;		/* Current client */
   job_t			*job,		/* Current job */
 			*next;		/* Next job */
@@ -90,7 +102,7 @@ main(int  argc,			/* I - Number of command-line arguments */
   struct sigaction	action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 #ifdef __sgi
-  FILE			*fp;		/* Fake lpsched lock file */
+  cups_file_t		*fp;		/* Fake lpsched lock file */
 #endif /* __sgi */
 
 
@@ -116,7 +128,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 	        * Absolute directory...
 		*/
 
-		strlcpy(ConfigurationFile, argv[i], sizeof(ConfigurationFile));
+		SetString(&ConfigurationFile, argv[i]);
               }
 	      else
 	      {
@@ -124,9 +136,11 @@ main(int  argc,			/* I - Number of command-line arguments */
 	        * Relative directory...
 		*/
 
-                getcwd(ConfigurationFile, sizeof(ConfigurationFile));
-		strlcat(ConfigurationFile, "/", sizeof(ConfigurationFile));
-		strlcat(ConfigurationFile, argv[i], sizeof(ConfigurationFile));
+                char current[1024];	/* Current directory */
+
+
+                getcwd(current, sizeof(current));
+		SetStringf(&ConfigurationFile, "%s/%s", current, argv[i]);
               }
 	      break;
 
@@ -149,47 +163,62 @@ main(int  argc,			/* I - Number of command-line arguments */
       usage();
     }
 
+  if (!ConfigurationFile)
+    SetString(&ConfigurationFile, CUPS_SERVERROOT "/cupsd.conf");
+
  /*
   * If the user hasn't specified "-f", run in the background...
   */
 
   if (!fg)
   {
+   /*
+    * Setup signal handlers for the parent...
+    */
+
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+    sigset(SIGUSR1, parent_handler);
+    sigset(SIGCHLD, parent_handler);
+
+    sigset(SIGHUP, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+    memset(&action, 0, sizeof(action));
+    sigemptyset(&action.sa_mask);
+    sigaddset(&action.sa_mask, SIGUSR1);
+    action.sa_handler = parent_handler;
+    sigaction(SIGUSR1, &action, NULL);
+    sigaction(SIGCHLD, &action, NULL);
+
+    sigemptyset(&action.sa_mask);
+    action.sa_handler = SIG_IGN;
+    sigaction(SIGHUP, &action, NULL);
+#else
+    signal(SIGUSR1, parent_handler);
+    signal(SIGCLD, parent_handler);
+
+    signal(SIGHUP, SIG_IGN);
+#endif /* HAVE_SIGSET */
+
     if (fork() > 0)
     {
      /*
-      * OK, wait for the child to startup and send us SIGUSR1...  We
-      * also need to ignore SIGHUP which might be sent by the init
-      * script to restart the scheduler...
+      * OK, wait for the child to startup and send us SIGUSR1 or to crash
+      * and the OS send us SIGCHLD...  We also need to ignore SIGHUP which
+      * might be sent by the init script to restart the scheduler...
       */
 
-#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-      sigset(SIGUSR1, sigusr1_handler);
+      for (; parent_signal == 0;)
+        sleep(1);
 
-      sigset(SIGHUP, SIG_IGN);
-#elif defined(HAVE_SIGACTION)
-      memset(&action, 0, sizeof(action));
-      sigemptyset(&action.sa_mask);
-      sigaddset(&action.sa_mask, SIGUSR1);
-      action.sa_handler = sigusr1_handler;
-      sigaction(SIGUSR1, &action, NULL);
-
-      sigemptyset(&action.sa_mask);
-      action.sa_handler = SIG_IGN;
-      sigaction(SIGHUP, &action, NULL);
-#else
-      signal(SIGUSR1, sigusr1_handler);
-
-      signal(SIGHUP, SIG_IGN);
-#endif /* HAVE_SIGSET */
-
-      if (wait(&i) < 0)
-        i = 0;
-
-      if (i == 0)
+      if (parent_signal == SIGUSR1)
         return (0);
 
-      if (i >= 256)
+      if (wait(&i) < 0)
+      {
+        perror("cupsd");
+	i = 1;
+      }
+      else if (i >= 256)
         fprintf(stderr, "cupsd: Child exited with status %d!\n", i / 256);
       else
         fprintf(stderr, "cupsd: Child exited on signal %d!\n", i);
@@ -232,9 +261,9 @@ main(int  argc,			/* I - Number of command-line arguments */
   */
 
   if (getenv("TZ") != NULL)
-    snprintf(TZ, sizeof(TZ), "TZ=%s", getenv("TZ"));
+    SetStringf(&TZ, "TZ=%s", getenv("TZ"));
   else
-    TZ[0] = '\0';
+    SetString(&TZ, "");
 
   tzset();
 
@@ -247,14 +276,43 @@ main(int  argc,			/* I - Number of command-line arguments */
   */
 
   getrlimit(RLIMIT_NOFILE, &limit);
-  if (limit.rlim_max > FD_SETSIZE)	/* Can't exceed size of FD set! */
-    MaxFDs = FD_SETSIZE;
+
+  if (limit.rlim_max > CUPS_MAX_FDS)
+    MaxFDs = CUPS_MAX_FDS;
   else
     MaxFDs = limit.rlim_max;
 
   limit.rlim_cur = MaxFDs;
+
   setrlimit(RLIMIT_NOFILE, &limit);
 
+ /*
+  * Allocate memory for the input and output sets...
+  */
+
+  SetSize   = (MaxFDs + 7) / 8;
+  InputSet  = (fd_set *)calloc(1, SetSize);
+  OutputSet = (fd_set *)calloc(1, SetSize);
+  input     = (fd_set *)calloc(1, SetSize);
+  output    = (fd_set *)calloc(1, SetSize);
+
+  if (InputSet == NULL || OutputSet == NULL || input == NULL || output == NULL)
+  {
+    syslog(LOG_LPR, "Unable to allocate memory for select() sets - exiting!");
+    return (1);
+  }
+
+ /*
+  * Read configuration...
+  */
+
+  if (!ReadConfiguration())
+  {
+    syslog(LOG_LPR, "Unable to read configuration file \'%s\' - exiting!",
+           ConfigurationFile);
+    return (1);
+  }
+
  /*
   * Catch hangup and child signals and ignore broken pipes...
   */
@@ -264,6 +322,7 @@ main(int  argc,			/* I - Number of command-line arguments */
     sigset(SIGHUP, sigterm_handler);
   else
     sigset(SIGHUP, sighup_handler);
+
   sigset(SIGPIPE, SIG_IGN);
   sigset(SIGTERM, sigterm_handler);
 #elif defined(HAVE_SIGACTION)
@@ -298,17 +357,6 @@ main(int  argc,			/* I - Number of command-line arguments */
   signal(SIGTERM, sigterm_handler);
 #endif /* HAVE_SIGSET */
 
- /*
-  * Read configuration...
-  */
-
-  if (!ReadConfiguration())
-  {
-    syslog(LOG_LPR, "Unable to read configuration file \'%s\' - exiting!",
-           ConfigurationFile);
-    return (1);
-  }
-
 #ifdef __sgi
  /*
   * Try to create a fake lpsched lock file if one is not already there.
@@ -316,7 +364,7 @@ main(int  argc,			/* I - Number of command-line arguments */
   * printing...
   */
 
-  if ((fp = fopen("/var/spool/lp/SCHEDLOCK", "a")) == NULL)
+  if ((fp = cupsFileOpen("/var/spool/lp/SCHEDLOCK", "w")) == NULL)
   {
     syslog(LOG_LPR, "Unable to create fake lpsched lock file "
                     "\"/var/spool/lp/SCHEDLOCK\"\' - %s!",
@@ -324,10 +372,10 @@ main(int  argc,			/* I - Number of command-line arguments */
   }
   else
   {
-    fclose(fp);
+    fchmod(cupsFileNumber(fp), 0644);
+    fchown(cupsFileNumber(fp), User, Group);
 
-    chmod("/var/spool/lp/SCHEDLOCK", 0644);
-    chown("/var/spool/lp/SCHEDLOCK", User, Group);
+    cupsFileClose(fp);
   }
 #endif /* __sgi */
 
@@ -357,6 +405,12 @@ main(int  argc,			/* I - Number of command-line arguments */
     setuid(User);
   }
 
+ /*
+  * Start any pending print jobs...
+  */
+
+  CheckJobs();
+
  /*
   * Loop forever...
   */
@@ -404,8 +458,8 @@ main(int  argc,			/* I - Number of command-line arguments */
     * times.
     */
 
-    input  = InputSet;
-    output = OutputSet;
+    memcpy(input, InputSet, SetSize);
+    memcpy(output, OutputSet, SetSize);
 
     for (i = NumClients, con = Clients; i > 0; i --, con ++)
       if (con->http.used > 0)
@@ -427,7 +481,7 @@ main(int  argc,			/* I - Number of command-line arguments */
       timeout.tv_usec = 0;
     }
 
-    if ((i = select(MaxFDs, &input, &output, NULL, &timeout)) < 0)
+    if ((i = select(MaxFDs, input, output, NULL, &timeout)) < 0)
     {
       char	s[16384],	/* String buffer */
 		*sptr;		/* Pointer into buffer */
@@ -452,7 +506,7 @@ main(int  argc,			/* I - Number of command-line arguments */
       sptr = s + 10;
 
       for (i = 0; i < MaxFDs; i ++)
-        if (FD_ISSET(i, &InputSet))
+        if (FD_ISSET(i, InputSet))
 	{
           snprintf(sptr, sizeof(s) - slen, " %d", i);
 	  slen += strlen(sptr);
@@ -466,7 +520,7 @@ main(int  argc,			/* I - Number of command-line arguments */
       sptr = s + 11;
 
       for (i = 0; i < MaxFDs; i ++)
-        if (FD_ISSET(i, &OutputSet))
+        if (FD_ISSET(i, OutputSet))
 	{
           snprintf(sptr, sizeof(s) - slen, " %d", i);
 	  slen += strlen(sptr);
@@ -491,7 +545,7 @@ main(int  argc,			/* I - Number of command-line arguments */
     }
 
     for (i = NumListeners, lis = Listeners; i > 0; i --, lis ++)
-      if (FD_ISSET(lis->fd, &input))
+      if (FD_ISSET(lis->fd, input))
         AcceptClient(lis);
 
     for (i = NumClients, con = Clients; i > 0; i --, con ++)
@@ -500,7 +554,7 @@ main(int  argc,			/* I - Number of command-line arguments */
       * Process the input buffer...
       */
 
-      if (FD_ISSET(con->http.fd, &input) || con->http.used)
+      if (FD_ISSET(con->http.fd, input) || con->http.used)
         if (!ReadClient(con))
 	{
 	  con --;
@@ -511,8 +565,8 @@ main(int  argc,			/* I - Number of command-line arguments */
       * Write data as needed...
       */
 
-      if (FD_ISSET(con->http.fd, &output) &&
-          (!con->pipe_pid || FD_ISSET(con->file, &input)))
+      if (FD_ISSET(con->http.fd, output) &&
+          (!con->pipe_pid || FD_ISSET(con->file, input)))
         if (!WriteClient(con))
 	{
 	  con --;
@@ -526,6 +580,9 @@ main(int  argc,			/* I - Number of command-line arguments */
       activity = time(NULL) - Timeout;
       if (con->http.activity < activity && !con->pipe_pid)
       {
+        LogMessage(L_DEBUG, "Closing client %d after %d seconds of inactivity...",
+	           con->http.fd, Timeout);
+
         CloseClient(con);
         con --;
         continue;
@@ -540,14 +597,14 @@ main(int  argc,			/* I - Number of command-line arguments */
     {
       next = job->next;
 
-      if (job->pipe && FD_ISSET(job->pipe, &input))
+      if (job->pipe && FD_ISSET(job->pipe, input))
       {
        /*
         * Clear the input bit to avoid updating the next job
 	* using the same status pipe file descriptor...
 	*/
 
-        FD_CLR(job->pipe, &input);
+        FD_CLR(job->pipe, input);
 
        /*
         * Read any status messages from the filters...
@@ -557,16 +614,23 @@ main(int  argc,			/* I - Number of command-line arguments */
       }
     }
 
+   /*
+    * Update CGI messages as needed...
+    */
+
+    if (CGIPipes[0] >= 0 && FD_ISSET(CGIPipes[0], input))
+      UpdateCGI();
+
    /*
     * Update the browse list as needed...
     */
 
     if (Browsing && BrowseProtocols)
     {
-      if (BrowseSocket >= 0 && FD_ISSET(BrowseSocket, &input))
+      if (BrowseSocket >= 0 && FD_ISSET(BrowseSocket, input))
         UpdateCUPSBrowse();
 
-      if (PollPipe >= 0 && FD_ISSET(PollPipe, &input))
+      if (PollPipe >= 0 && FD_ISSET(PollPipe, input))
         UpdatePolling();
 
 #ifdef HAVE_LIBSLP
@@ -661,8 +725,54 @@ CatchChildSignals(void)
 }
 
 
+/*
+ * 'ClearString()' - Clear a string.
+ */
+
+void
+ClearString(char **s)			/* O - String value */
+{
+  if (s && *s)
+  {
+    free(*s);
+    *s = NULL;
+  }
+}
+
+
+/*
+ * 'HoldSignals()' - Hold child and termination signals.
+ */
+
+void
+HoldSignals(void)
+{
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  sigset_t		newmask;	/* New POSIX signal mask */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
+
+
+  holdcount ++;
+  if (holdcount > 1)
+    return;
+
+#ifdef HAVE_SIGSET
+  sighold(SIGTERM);
+  sighold(SIGCHLD);
+#elif defined(HAVE_SIGACTION)
+  sigemptyset(&newmask);
+  sigaddset(&newmask, SIGTERM);
+  sigaddset(&newmask, SIGCHLD);
+  sigprocmask(SIG_BLOCK, &newmask, &holdmask);
+#endif /* HAVE_SIGSET */
+}
+
+
 /*
  * 'IgnoreChildSignals()' - Ignore SIGCHLD signals...
+ *
+ * We don't really ignore them, we set the signal handler to SIG_DFL,
+ * since some OS's rely on signals for the wait4() function to work.
  */
 
 void
@@ -672,21 +782,113 @@ IgnoreChildSignals(void)
   struct sigaction	action;		/* Actions for POSIX signals */
 #endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
+
 #ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-  sigset(SIGCHLD, SIG_IGN);
+  sigset(SIGCHLD, SIG_DFL);
 #elif defined(HAVE_SIGACTION)
   memset(&action, 0, sizeof(action));
 
   sigemptyset(&action.sa_mask);
   sigaddset(&action.sa_mask, SIGCHLD);
-  action.sa_handler = SIG_IGN;
+  action.sa_handler = SIG_DFL;
   sigaction(SIGCHLD, &action, NULL);
 #else
-  signal(SIGCLD, SIG_IGN);	/* No, SIGCLD isn't a typo... */
+  signal(SIGCLD, SIG_DFL);	/* No, SIGCLD isn't a typo... */
 #endif /* HAVE_SIGSET */
 }
 
 
+/*
+ * 'ReleaseSignals()' - Release signals for delivery.
+ */
+
+void
+ReleaseSignals(void)
+{
+  holdcount --;
+  if (holdcount > 0)
+    return;
+
+#ifdef HAVE_SIGSET
+  sigrelse(SIGTERM);
+  sigrelse(SIGCHLD);
+#elif defined(HAVE_SIGACTION)
+  sigprocmask(SIG_SETMASK, &holdmask, NULL);
+#endif /* HAVE_SIGSET */
+}
+
+
+/*
+ * 'SetString()' - Set a string value.
+ */
+
+void
+SetString(char       **s,		/* O - New string */
+          const char *v)		/* I - String value */
+{
+  if (!s || *s == v)
+    return;
+
+  if (*s)
+    free(*s);
+
+  if (v)
+    *s = strdup(v);
+  else
+    *s = NULL;
+}
+
+
+/*
+ * 'SetStringf()' - Set a formatted string value.
+ */
+
+void
+SetStringf(char       **s,		/* O - New string */
+           const char *f,		/* I - Printf-style format string */
+	   ...)				/* I - Additional args as needed */
+{
+  char		v[1024];		/* Formatting string value */
+  va_list	ap;			/* Argument pointer */
+  char		*olds;			/* Old string */
+
+
+  if (!s)
+    return;
+
+  olds = *s;
+
+  if (f)
+  {
+    va_start(ap, f);
+    vsnprintf(v, sizeof(v), f, ap);
+    va_end(ap);
+
+    *s = strdup(v);
+  }
+  else
+    *s = NULL;
+
+  if (olds)
+    free(olds);
+}
+
+
+/*
+ * 'parent_handler()' - Catch USR1/CHLD signals...
+ */
+
+static void
+parent_handler(int sig)		/* I - Signal */
+{
+ /*
+  * Store the signal we got from the OS and return...
+  */
+
+  parent_signal = sig;
+}
+
+
 /*
  * 'sigchld_handler()' - Handle 'child' signals from old processes.
  */
@@ -810,7 +1012,7 @@ sighup_handler(int sig)	/* I - Signal number */
 {
   (void)sig;
 
-  NeedReload = TRUE;
+  NeedReload = RELOAD_ALL;
 
 #ifdef HAVE_SIGSET
   sigset(SIGHUP, sighup_handler);
@@ -844,39 +1046,9 @@ sigterm_handler(int sig)		/* I - Signal */
   * Close all network clients and stop all jobs...
   */
 
-  CloseAllClients();
-  StopListening();
-  StopPolling();
-  StopBrowsing();
+  StopServer();
 
-  if (Clients != NULL)
-    free(Clients);
-
-  FreeAllJobs();
-
-  if (AccessFile != NULL)
-    fclose(AccessFile);
-
-  if (ErrorFile != NULL)
-    fclose(ErrorFile);
-
-  if (PageFile != NULL)
-    fclose(PageFile);
-
-  DeleteAllLocations();
-
-  DeleteAllClasses();
-
-  if (Devices)
-    ippDelete(Devices);
-
-  if (PPDs)
-    ippDelete(PPDs);
-
-  DeleteAllPrinters();
-
-  if (MimeDatabase != NULL)
-    mimeDelete(MimeDatabase);
+  StopAllJobs();
 
 #ifdef __sgi
  /*
@@ -894,17 +1066,6 @@ sigterm_handler(int sig)		/* I - Signal */
 }
 
 
-/*
- * 'sigusr1_handler()' - Catch USR1 signals...
- */
-
-static void
-sigusr1_handler(int sig)		/* I - Signal */
-{
-  (void)sig;	/* remove compiler warnings... */
-}
-
-
 /*
  * 'usage()' - Show scheduler usage.
  */
@@ -912,11 +1073,11 @@ sigusr1_handler(int sig)		/* I - Signal */
 static void
 usage(void)
 {
-  fputs("Usage: cupsd [-c config-file] [-f]\n", stderr);
+  fputs("Usage: cupsd [-c config-file] [-f] [-F]\n", stderr);
   exit(1);
 }
 
 
 /*
- * End of "$Id: main.c,v 1.85 2002/12/17 19:00:17 swdev Exp $".
+ * End of "$Id: main.c,v 1.104 2003/05/09 15:11:13 mike Exp $".
  */
