@@ -1,5 +1,5 @@
 /*
- * "$Id: mime.h,v 1.8 2002/12/17 19:00:17 swdev Exp $"
+ * "$Id: mime.h,v 1.11 2003/04/07 18:03:28 mike Exp $"
  *
  *   MIME type/conversion database definitions for the Common UNIX Printing System (CUPS).
  *
@@ -22,10 +22,11 @@
  *         WWW: http://www.cups.org
  */
 
-#ifndef _MIME_H_
-#  define _MIME_H_
+#ifndef _CUPS_MIME_H_
+#  define _CUPS_MIME_H_
 
 #  include <cups/ipp.h>
+#  include "file.h"
 
 
 /*
@@ -64,7 +65,8 @@ typedef enum
   MIME_MAGIC_SHORT,	/* Short/16-bit word matches */
   MIME_MAGIC_INT,	/* Integer/32-bit word matches */
   MIME_MAGIC_LOCALE,	/* Current locale matches string */
-  MIME_MAGIC_CONTAINS	/* File contains a string */
+  MIME_MAGIC_CONTAINS,	/* File contains a string */
+  MIME_MAGIC_ISTRING	/* Case-insensitive string matches */
 } mime_op_t;
 
 typedef struct mime_magic_str		/**** MIME Magic Data ****/
@@ -126,7 +128,8 @@ extern mime_t		*mimeNew(void);
 
 extern mime_type_t	*mimeAddType(mime_t *mime, const char *super, const char *type);
 extern int		mimeAddTypeRule(mime_type_t *mt, const char *rule);
-extern mime_type_t	*mimeFileType(mime_t *mime, const char *pathname);
+extern mime_type_t	*mimeFileType(mime_t *mime, const char *pathname,
+			              int *compression);
 extern mime_type_t	*mimeType(mime_t *mime, const char *super, const char *type);
 
 extern mime_filter_t	*mimeAddFilter(mime_t *mime, mime_type_t *src, mime_type_t *dst,
@@ -137,8 +140,8 @@ extern mime_filter_t	*mimeFilter(mime_t *mime, mime_type_t *src, mime_type_t *ds
 #  ifdef _cplusplus
 }
 #  endif /* _cplusplus */
-#endif /* !_MIME_H_ */
+#endif /* !_CUPS_MIME_H_ */
 
 /*
- * End of "$Id: mime.h,v 1.8 2002/12/17 19:00:17 swdev Exp $".
+ * End of "$Id: mime.h,v 1.11 2003/04/07 18:03:28 mike Exp $".
  */
