@@ -1,5 +1,5 @@
 /*
- * "$Id: testmime.c,v 1.9 2002/12/17 19:00:18 swdev Exp $"
+ * "$Id: testmime.c,v 1.12 2003/05/23 20:40:48 mike Exp $"
  *
  *   MIME test program for the Common UNIX Printing System (CUPS).
  *
@@ -51,9 +51,11 @@ int					/* O - Exit status */
 main(int  argc,				/* I - Number of command-line args */
      char *argv[])			/* I - Command-line arguments */
 {
-  int		i;			/* Looping var */
+  int		i, j;			/* Looping vars */
+  const char	*filter_path;		/* Filter path */
   char		super[MIME_MAX_SUPER],	/* Super-type name */
 		type[MIME_MAX_TYPE];	/* Type name */
+  int		compression;		/* Compression of file */
   mime_t	*mime;			/* MIME database */
   mime_type_t	*src,			/* Source type */
 		*dst,			/* Destination type */
@@ -62,9 +64,10 @@ main(int  argc,				/* I - Number of command-line args */
   int		num_filters;		/* Number of filters for the file */
 
 
-  mime = NULL;
-  src  = NULL;
-  dst  = NULL;
+  mime        = NULL;
+  src         = NULL;
+  dst         = NULL;
+  filter_path = "../filter";
 
   for (i = 1; i < argc; i ++)
     if (strcmp(argv[i], "-d") == 0)
@@ -72,17 +75,25 @@ main(int  argc,				/* I - Number of command-line args */
       i ++;
 
       if (i < argc)
-        mime = mimeLoad(argv[i], "../filter");
+        mime = mimeLoad(argv[i], filter_path);
+    }
+    else if (strcmp(argv[i], "-f") == 0)
+    {
+      i ++;
+
+      if (i < argc)
+        filter_path = argv[i];
     }
     else if (src == NULL)
     {
       if (!mime)
-	mime = mimeLoad("../conf", "../filter");
+	mime = mimeLoad("../conf", filter_path);
 
-      src = mimeFileType(mime, argv[i]);
+      src = mimeFileType(mime, argv[i], &compression);
 
       if (src != NULL)
-	printf("%s: %s/%s\n", argv[i], src->super, src->type);
+	printf("%s: %s/%s%s\n", argv[i], src->super, src->type,
+	       compression ? " (gzipped)" : "");
       else
       {
 	printf("%s: unknown\n", argv[i]);
@@ -105,18 +116,18 @@ main(int  argc,				/* I - Number of command-line args */
       }
       else
       {
-	for (i = 0; i < num_filters; i ++)
-	  if (i < (num_filters - 1))
-	    printf("%s | ", filters[i].filter);
+	for (j = 0; j < num_filters; j ++)
+	  if (j < (num_filters - 1))
+	    printf("%s | ", filters[j].filter);
 	  else
-	    puts(filters[i].filter);
+	    puts(filters[j].filter);
 
         free(filters);
       }
     }
 
   if (!mime)
-    mime = mimeLoad("../conf", "../filter");
+    mime = mimeLoad("../conf", filter_path);
 
   if (src == NULL)
   {
@@ -232,5 +243,5 @@ print_rules(mime_magic_t *rules)	/* I - Rules to print */
 
 
 /*
- * End of "$Id: testmime.c,v 1.9 2002/12/17 19:00:18 swdev Exp $".
+ * End of "$Id: testmime.c,v 1.12 2003/05/23 20:40:48 mike Exp $".
  */
