@@ -1,5 +1,5 @@
 /*
- * "$Id: cancel.c,v 1.28 2002/12/17 19:00:20 swdev Exp $"
+ * "$Id: cancel.c,v 1.31 2003/04/28 19:26:39 mike Exp $"
  *
  *   "cancel" command for the Common UNIX Printing System (CUPS).
  *
@@ -54,7 +54,9 @@ main(int  argc,			/* I - Number of command-line arguments */
   int		num_dests;	/* Number of destinations */
   cups_dest_t	*dests;		/* Destinations */
   char		*dest,		/* Destination printer */
-		*job;		/* Job ID pointer */
+		*job,		/* Job ID pointer */
+		*user;		/* Cancel jobs for a user */
+  int		purge;		/* Purge or cancel jobs? */
   char		uri[1024];	/* Printer or job URI */
   ipp_t		*request;	/* IPP request */
   ipp_t		*response;	/* IPP response */
@@ -67,8 +69,10 @@ main(int  argc,			/* I - Number of command-line arguments */
   */
 
   op         = IPP_CANCEL_JOB;
+  purge      = 0;
   job_id     = 0;
   dest       = NULL;
+  user       = NULL;
   http       = NULL;
   num_dests  = 0;
   dests      = NULL;
@@ -82,7 +86,7 @@ main(int  argc,			/* I - Number of command-line arguments */
       switch (argv[i][1])
       {
         case 'E' : /* Encrypt */
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
 	    cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
 
 	    if (http)
@@ -90,11 +94,12 @@ main(int  argc,			/* I - Number of command-line arguments */
 #else
             fprintf(stderr, "%s: Sorry, no encryption support compiled in!\n",
 	            argv[0]);
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 	    break;
 
         case 'a' : /* Cancel all jobs */
-	    op = IPP_PURGE_JOBS;
+	    purge = 1;
+	    op    = IPP_PURGE_JOBS;
 	    break;
 
         case 'h' : /* Connect to host */
@@ -118,8 +123,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    break;
 
         case 'u' : /* Username */
+	    op = IPP_PURGE_JOBS;
+
 	    if (argv[i][2] != '\0')
-	      cupsSetUser(argv[i] + 2);
+	      user = argv[i] + 2;
 	    else
 	    {
 	      i ++;
@@ -130,7 +137,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 		return (1);
               }
 	      else
-		cupsSetUser(argv[i]);
+		user = argv[i];
 	    }
 	    break;
 
@@ -254,8 +261,18 @@ main(int  argc,			/* I - Number of command-line arguments */
 	             uri);
       }
 
-      ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
-                   "requesting-user-name", NULL, cupsUser());
+      if (user)
+      {
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                     "requesting-user-name", NULL, user);
+	ippAddBoolean(request, IPP_TAG_OPERATION, "my-jobs", 1);
+      }
+      else
+	ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                     "requesting-user-name", NULL, cupsUser());
+
+      if (op == IPP_PURGE_JOBS)
+	ippAddBoolean(request, IPP_TAG_OPERATION, "purge-jobs", purge);
 
      /*
       * Do the request and get back a response...
@@ -283,10 +300,85 @@ main(int  argc,			/* I - Number of command-line arguments */
       ippDelete(response);
     }
 
+  if (num_dests == 0 && op == IPP_PURGE_JOBS)
+  {
+   /*
+    * Open a connection to the server...
+    */
+
+    if (http == NULL)
+      if ((http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                             cupsEncryption())) == NULL)
+      {
+	fputs("cancel: Unable to contact server!\n", stderr);
+	return (1);
+      }
+
+   /*
+    * Build an IPP request, which requires the following
+    * attributes:
+    *
+    *    attributes-charset
+    *    attributes-natural-language
+    *    printer-uri + job-id *or* job-uri
+    *    [requesting-user-name]
+    */
+
+    request = ippNew();
+
+    request->request.op.operation_id = op;
+    request->request.op.request_id   = 1;
+
+    language = cupsLangDefault();
+
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
+              	 "attributes-charset", NULL, cupsLangEncoding(language));
+
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
+                 "attributes-natural-language", NULL, language->language);
+
+    ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
+	         "printer-uri", NULL, "ipp://localhost/printers/");
+
+    if (user)
+    {
+      ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                   "requesting-user-name", NULL, user);
+      ippAddBoolean(request, IPP_TAG_OPERATION, "my-jobs", 1);
+    }
+    else
+      ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+                   "requesting-user-name", NULL, cupsUser());
+
+    ippAddBoolean(request, IPP_TAG_OPERATION, "purge-jobs", purge);
+
+   /*
+    * Do the request and get back a response...
+    */
+
+    response = cupsDoRequest(http, request, "/admin/");
+
+    if (response == NULL ||
+        response->request.status.status_code > IPP_OK_CONFLICT)
+    {
+      fprintf(stderr, "cancel: %s failed: %s\n",
+	      op == IPP_PURGE_JOBS ? "purge-jobs" : "cancel-job",
+              response ? ippErrorString(response->request.status.status_code) :
+		         ippErrorString(cupsLastError()));
+
+      if (response)
+	ippDelete(response);
+
+      return (1);
+    }
+
+    ippDelete(response);
+  }
+
   return (0);
 }
 
 
 /*
- * End of "$Id: cancel.c,v 1.28 2002/12/17 19:00:20 swdev Exp $".
+ * End of "$Id: cancel.c,v 1.31 2003/04/28 19:26:39 mike Exp $".
  */
