@@ -1,5 +1,5 @@
 /*
- * "$Id: lp.c,v 1.39 2002/12/17 19:00:20 swdev Exp $"
+ * "$Id: lp.c,v 1.44 2003/05/09 18:50:42 mike Exp $"
  *
  *   "lp" command for the Common UNIX Printing System (CUPS).
  *
@@ -23,8 +23,10 @@
  *
  * Contents:
  *
- *   main()       - Parse options and send files for printing.
- *   sighandler() - Signal catcher for when we print from stdin...
+ *   main()          - Parse options and send files for printing.
+ *   restart_job()   - Restart a job.
+ *   set_job_attrs() - Set job attributes.
+ *   sighandler()    - Signal catcher for when we print from stdin...
  */
 
 /*
@@ -50,6 +52,7 @@
 
 void	sighandler(int);
 #endif /* !WIN32 */
+int	restart_job(int job_id);
 int	set_job_attrs(int job_id, int num_options, cups_option_t *options);
 
 
@@ -121,16 +124,16 @@ main(int  argc,		/* I - Number of command-line arguments */
   job_id      = 0;
 
   for (i = 1; i < argc; i ++)
-    if (argv[i][0] == '-')
+    if (argv[i][0] == '-' && argv[i][1])
       switch (argv[i][1])
       {
         case 'E' : /* Encrypt */
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_SSL
 	    cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
 #else
             fprintf(stderr, "%s: Sorry, no encryption support compiled in!\n",
 	            argv[0]);
-#endif /* HAVE_LIBSSL */
+#endif /* HAVE_SSL */
 	    break;
 
         case 'c' : /* Copy to spool dir (always enabled) */
@@ -379,6 +382,17 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    else if (strcmp(val, "immediate") == 0)
               num_options = cupsAddOption("job-priority", "100",
 	                                  num_options, &options);
+	    else if (strcmp(val, "restart") == 0)
+	    {
+	      if (job_id < 1)
+	      {
+	        fputs("lp: Need job ID (-i) before \"-H restart\"!\n", stderr);
+		return (1);
+	      }
+
+	      if (restart_job(job_id))
+	        return (1);
+	    }
 	    else
               num_options = cupsAddOption("job-hold-until", val,
 	                                  num_options, &options);
@@ -438,6 +452,17 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    fprintf(stderr, "lp: Unknown option \'%c\'!\n", argv[i][1]);
 	    return (1);
       }
+    else if (!strcmp(argv[i], "-"))
+    {
+      if (num_files || job_id)
+      {
+        fputs("lp: Error - cannot print from stdin if files or a job ID are "
+	      "provided!\n", stderr);
+	return (1);
+      }
+
+      break;
+    }
     else if (num_files < 1000 && job_id == 0)
     {
      /*
@@ -481,23 +506,25 @@ main(int  argc,		/* I - Number of command-line arguments */
     if (num_dests == 0)
       num_dests = cupsGetDests(&dests);
 
-    for (j = 0, dest = dests; j < num_dests; j ++, dest ++)
-      if (dest->is_default)
-      {
-	printer = dests[j].name;
+    if ((dest = cupsGetDest(NULL, NULL, num_dests, dests)) != NULL)
+    {
+      printer = dest->name;
 
-	for (j = 0; j < dest->num_options; j ++)
-	  if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
-	    num_options = cupsAddOption(dest->options[j].name,
-		                        dest->options[j].value,
-					num_options, &options);
-        break;
-      }
+      for (j = 0; j < dest->num_options; j ++)
+	if (cupsGetOption(dest->options[j].name, num_options, options) == NULL)
+	  num_options = cupsAddOption(dest->options[j].name,
+		                      dest->options[j].value,
+				      num_options, &options);
+    }
   }
 
   if (printer == NULL)
   {
-    fputs("lp: error - no default destination available.\n", stderr);
+    if (cupsLastError() >= IPP_BAD_REQUEST)
+      fputs("lp: error - scheduler not responding!\n", stderr);
+    else
+      fputs("lp: error - no default destination available.\n", stderr);
+
     return (1);
   }
 
@@ -571,6 +598,65 @@ main(int  argc,		/* I - Number of command-line arguments */
 }
 
 
+/*
+ * 'restart_job()' - Restart a job.
+ */
+
+int					/* O - Exit status */
+restart_job(int job_id)			/* I - Job ID */
+{
+  http_t	*http;			/* HTTP connection to server */
+  ipp_t		*request,		/* IPP request */
+		*response;		/* IPP response */
+  cups_lang_t	*language;		/* Language for request */
+  char		uri[HTTP_MAX_URI];	/* URI for job */
+
+
+  http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
+
+  language = cupsLangDefault();
+
+  request = ippNew();
+  request->request.op.operation_id = IPP_RESTART_JOB;
+  request->request.op.request_id   = 1;
+
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_CHARSET,
+               "attributes-charset", NULL, cupsLangEncoding(language));
+
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_LANGUAGE,
+               "attributes-natural-language", NULL, language->language);
+
+  sprintf(uri, "ipp://localhost/jobs/%d", job_id);
+
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_URI,
+               "job-uri", NULL, uri);
+
+  ippAddString(request, IPP_TAG_OPERATION, IPP_TAG_NAME,
+               "requesting-user-name", NULL, cupsUser());
+
+  if ((response = cupsDoRequest(http, request, "/jobs")) != NULL)
+  {
+    if (response->request.status.status_code > IPP_OK_CONFLICT)
+    {
+      fprintf(stderr, "lp: restart-job failed: %s\n",
+              ippErrorString(response->request.status.status_code));
+      ippDelete(response);
+      return (1);
+    }
+
+    ippDelete(response);
+  }
+  else
+  {
+    fprintf(stderr, "lp: restart-job failed: %s\n",
+            ippErrorString(cupsLastError()));
+    return (1);
+  }
+
+  return (0);
+}
+
+
 /*
  * 'set_job_attrs()' - Set job attributes.
  */
@@ -658,5 +744,5 @@ sighandler(int s)	/* I - Signal number */
 
 
 /*
- * End of "$Id: lp.c,v 1.39 2002/12/17 19:00:20 swdev Exp $".
+ * End of "$Id: lp.c,v 1.44 2003/05/09 18:50:42 mike Exp $".
  */
