@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# "$Id: run-stp-tests.sh,v 1.14 2002/12/17 19:00:25 swdev Exp $"
+# "$Id: run-stp-tests.sh,v 1.18 2003/04/23 18:38:37 mike Exp $"
 #
 #   Perform the complete set of IPP compliance tests specified in the
 #   CUPS Software Test Plan.
@@ -47,9 +47,10 @@ echo ""
 echo "OK, now that we have the Dirty Harry quote out of the way, please"
 echo "choose the type of test you wish to perform:"
 echo ""
-echo "1 - Basic conformance test with no load testing (all systems)"
-echo "2 - Basic conformance test with some load testing (minimum 256MB VM)"
-echo "3 - Basic conformance test with extreme load testing (minimum 1024MB VM)"
+echo "1 - Basic conformance test, no load testing (all systems)"
+echo "2 - Basic conformance test, some load testing (minimum 256MB VM, 50MB disk)"
+echo "3 - Basic conformance test, extreme load testing (minimum 1GB VM, 500MB disk)"
+echo "4 - Basic conformance test, torture load testing (minimum 2GB VM, 1GB disk)"
 echo ""
 echo "Please enter the number of the test you wish to perform:"
 
@@ -68,6 +69,12 @@ case "$testtype" in
 		nprinters2=1000
 		pjobs=100
 		;;
+	4)
+		echo "Running the torture tests (4)"
+		nprinters1=10000
+		nprinters2=20000
+		pjobs=200
+		;;
 	*)
 		echo "Running the timid tests (1)"
 		nprinters1=0
@@ -121,7 +128,6 @@ ln -s $root/filter/rastertoepson /tmp/$user/bin/filter
 ln -s $root/filter/rastertohp /tmp/$user/bin/filter
 ln -s $root/filter/texttops /tmp/$user/bin/filter
 ln -s $root/pdftops/pdftops /tmp/$user/bin/filter
-ln -s $root/pstoraster/pstoraster /tmp/$user/bin/filter
 
 ln -s $root/data/classified /tmp/$user/share/banners
 ln -s $root/data/confidential /tmp/$user/share/banners
@@ -133,7 +139,6 @@ ln -s $root/data /tmp/$user/share/charsets
 ln -s $root/data /tmp/$user/share
 ln -s $root/fonts /tmp/$user/share
 ln -s $root/ppd/*.ppd /tmp/$user/share/model
-ln -s $root/pstoraster /tmp/$user/share
 ln -s $root/templates /tmp/$user/share
 
 #
@@ -142,6 +147,7 @@ ln -s $root/templates /tmp/$user/share
 
 cat >/tmp/$user/cupsd.conf <<EOF
 Browsing Off
+FileDevice yes
 Listen 127.0.0.1:$port
 User $user
 ServerRoot /tmp/$user
@@ -151,10 +157,11 @@ FontPath /tmp/$user/share/fonts
 DocumentRoot $root/doc
 RequestRoot /tmp/$user/spool
 TempDir /tmp/$user/spool/temp
+MaxLogSize 0
 AccessLog /tmp/$user/log/access_log
 ErrorLog /tmp/$user/log/error_log
 PageLog /tmp/$user/log/page_log
-LogLevel debug
+LogLevel debug2
 PreserveJobHistory Yes
 <Location />
 Order deny,allow
@@ -263,7 +270,7 @@ export HOME
 
 echo "Starting scheduler..."
 
-../scheduler/cupsd -c /tmp/$user/cupsd.conf -f &
+../scheduler/cupsd -c /tmp/$user/cupsd.conf -f >/tmp/$user/log/debug_log &
 cupsd=$!
 
 echo "Scheduler is PID $cupsd; run debugger now if you need to."
@@ -421,5 +428,5 @@ echo "    $pdffile"
 echo ""
 
 #
-# End of "$Id: run-stp-tests.sh,v 1.14 2002/12/17 19:00:25 swdev Exp $"
+# End of "$Id: run-stp-tests.sh,v 1.18 2003/04/23 18:38:37 mike Exp $"
 #
